!  Copyright (C) 2019-2021, respective authors of MCFM.
!
!  This program is free software: you can redistribute it and/or modify it under
!  the terms of the GNU General Public License as published by the Free Software
!  Foundation, either version 3 of the License, or (at your option) any later
!  version.
!
!  This program is distributed in the hope that it will be useful, but WITHOUT ANY
!  WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS FOR A
!  PARTICULAR PURPOSE. See the GNU General Public License for more details.
!
!  You should have received a copy of the GNU General Public License along with
!  this program. If not, see <http://www.gnu.org/licenses/>
 
!--- This is a driver function for photon isolation, the relevant photon isolation 
!--- critera are selected and applied 

      function iso(p,phot_id,isub,nd)
       implicit none
      include 'src/Inc/types.f'
      logical:: iso
      
      include 'src/Inc/constants.f'
      include 'src/Inc/nf.f'
      include 'src/Inc/mxpart.f'
      include 'src/Inc/cplx.h'
      include 'src/Inc/frag.f'
      include 'src/Inc/useet.f'
      include 'src/Inc/phot_dip.f'
      include 'src/Inc/z_dip.f'
      include 'src/Inc/lastphot.f'
      include 'src/Inc/mpicommon.f'
      include 'src/Inc/first.f'
      real(dp):: p(mxpart,4)
      integer:: isub,nd,imode
      integer:: phot_id ! refers to which photon we are isolating
      logical:: photo_iso_phys,photo_iso_z!,photo_iso,iso_old

c--- imode: set the behaviour of the isolation requirement (default: imode=0)
c---   imode=0 : scaling cut for epsilon_h < 1, fixed cut for epsilon_h > 1
c---   imode=1 : scaling cut for all epsilon_h
c---   imode=2 : fixed cut for all epsilon_h
      imode=0     

      iso=.true.

! Check if no isolation required
      if ((abs(cone_ang) < 1.e-4_dp).or.(abs(epsilon_h) < 1.e-4_dp)) then
        if (first) then
!$omp master
        if (rank == 0) then
        write(6,*)'****************************************************'
        write(6,*)'*                                                  *'
        write(6,*)'*         No photon isolation cuts applied         *'
        write(6,*)'*                                                  *'
        write(6,*)'****************************************************'
        endif
!$omp end master
        first=.false.
      endif
        return
      endif

c--- for imode=0, decide which cut to use depending on epsilon_h
!--   epsilon_h < 1 : epsilon_h corresponds to a pt fraction i.e. pt(had) < epsilon_h pt_gamma 
!--   epsilon_h > 1 : treat it as an E_t max in cone   i.e. pt(had) < epsilon_h 
       if (imode == 0) then
         if (epsilon_h < 0.9999_dp) then
         imode=1
         else
         imode=2
         endif
       endif

!===== NEW ISOLATION : WORK OUT WHICH ROUTINE TO CALL 
 
!========== are we doing fragmentation and is this the
!========== photon produced by fragmentation? 
       if((fragint_mode) .and. (phot_id == lastphot)) then 
          iso=photo_iso_z(p,phot_id,z_frag,imode,isub) 
!========== are we doing a photon dipole and is this the
!========== photon produced by fragmentation? 
       elseif(phot_dip(nd) .and. (phot_id == lastphot)) then 
          iso=photo_iso_z(p,phot_id,z_dip(nd),imode,isub) 
!========== we are in default mode, BORN, virt, real, non-photon dipole 
       else
          iso=photo_iso_phys(p,phot_id,imode,isub) 
       endif

!===== OLD ISOLATION 
c       iso_old=photo_iso(p,isub,phot_dip(nd),phot_id,nd,imode)       
c       if (iso .neqv. iso_old) then
c         write(6,*)'WARNING:',iso,'(new) vs',iso_old,' (old) for nd=',nd
c       else
c         write(6,*) 'OKAY'
c       endif
       
c--- write out isolation parameters
       if    (first .and. (imode == 1)) then 
!$omp master
        if (rank == 0) then
        write(6,*)'************** Photons Isolated     ****************'
        write(6,*)'*                                                  *'
        write(6,99)'*    E_t(had) in cone',cone_ang,' < ',epsilon_h,
     &   ' E_t(phot)     *'
        write(6,*)'*                                                  *'
        write(6,*)'****************************************************'
        endif
!$omp end master
        first=.false.
      elseif (first .and. (imode == 2)) then
!$omp master
        if (rank == 0) then
        write(6,*)'************** Photons Isolated     ****************'
        write(6,*)'*                                                  *'
        write(6,96)'* E_t (had) in cone',cone_ang,' < ',epsilon_h,
     &   'GeV    *'
        write(6,*)'*                                                  *'
        write(6,*)'****************************************************'
        endif
!$omp end master
        first=.false.
      endif
      
!      endif

      return 

 99   format(1x,a21,f6.2,a3,f6.2,a16)
 96   format(1x,a19,f6.2,a4,f6.2,a17)
      end
