!  Copyright (C) 2019-2021, respective authors of MCFM.
!
!  This program is free software: you can redistribute it and/or modify it under
!  the terms of the GNU General Public License as published by the Free Software
!  Foundation, either version 3 of the License, or (at your option) any later
!  version.
!
!  This program is distributed in the hope that it will be useful, but WITHOUT ANY
!  WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS FOR A
!  PARTICULAR PURPOSE. See the GNU General Public License for more details.
!
!  You should have received a copy of the GNU General Public License along with
!  this program. If not, see <http://www.gnu.org/licenses/>
 
      subroutine checkjets(jetsfound,qfinal,isub,failed)
      implicit none
      include 'src/Inc/types.f'
c--- performs checks on the jets that are found by the clustering algorithm,
c--- to ensure that the correct number of heavy quark jets is found,
c--- in the right invariant mass range
c--- given integer:: 'jetsfound' jets with momenta 'qfinal', 'isub'
c--- returns logical:: 'failed'
      
      include 'src/Inc/bbproc.f'
      include 'src/Inc/constants.f'
      include 'src/Inc/nf.f'
      include 'src/Inc/mxpart.f'
      include 'src/Inc/cplx.h'
      include 'src/Inc/jetlabel.f'
      include 'src/Inc/limits.f'
      include 'src/Inc/kprocess.f'
      include 'src/Inc/removebr.f'
      include 'src/Inc/npart.f'
      include 'src/Inc/nproc.f'
      real(dp):: qfinal(mxpart,4),m56,m57,m67
      logical:: failed
      integer:: countb,jetsfound,nbq,nba,isub
      
      failed=.false.

      if (nproc == 1610 .or. nproc == 1650) then
          ! we no longer rely on this routine, since at
          ! NNLO in kbq_tpq and NLO in kbq_tpq_jet there
          ! are contributions with different b final states
          return
      endif

c--- check that particle 5 is a b for H+b, W+c, Z+Q and W+b+jet processes
c--- also check for t-channel single top when the BR is not removed
      if ( (kcase==kH_1jet)
     & .or.(kcase==kW_cjet) .or. (kcase==kWcjet0)
     & .or.(kcase==kgQ__ZQ) .or. (kcase==kW_bjet)
     & .or.(kcase==kZ_bjet)
     & .or.(kcase==ktopanom)
     & .or.((kcase==kbq_tpq) .and. (removebr .eqv. .false.))
     & .or.((kcase==kttdkay) .and. (removebr .eqv. .false.))
     & .or.((kcase==kW_twdk) .and. (removebr .eqv. .false.))
     & .or.((kcase==kWtdkay) .and. (removebr .eqv. .false.)) ) then
        countb=0
        if ((jetsfound >= 1) .and. ((jetlabel(1) == 'bq')
     &    .or. (jetlabel(1) == 'ba'))) countb=1
        if ((jetsfound >= 2) .and. ((jetlabel(2) == 'bq')
     &    .or. (jetlabel(2) == 'ba'))) countb=countb+1
        if ((jetsfound >= 3) .and. ((jetlabel(3) == 'bq')
     &    .or. (jetlabel(3) == 'ba'))) countb=countb+1
        if ((jetsfound == 1) .and. (countb == 0)) failed=.true.
        if (   (     (kcase==kbq_tpq) .or. (kcase==kttdkay)
     &          .or. (kcase==kW_bjet) .or. (kcase==kZ_bjet)
     &          .or. (kcase==ktopanom) )
     &   .and. (countb < 1) ) failed=.true.
        if ((nproc == 132) .and. (jetsfound == 2)
     &      .and. (countb .ne. 1)) failed=.true.
        if ((nproc == 133) .and. (jetsfound == 2)
     &      .and. (countb .ne. 2)) failed=.true.
        if ((nproc == 342) .and. (jetsfound == 2)
     &      .and. (countb .ne. 1) .and. (isub == 0)) failed=.true.
      endif
            
c--- check that 5 and 6 are b and b-bar (if appropriate)
      if (bbproc) then
        call getbs(qfinal,nbq,nba)
        if ((nbq == 0) .or. (nba == 0)) failed=.true.    
      endif

c--- perform m56 mass cut if there are 2 or more jets found
c--- and there are at least 6 particles in the final state
      if ((npart > 3) .and. (jetsfound >= 2)) then
        m56=(qfinal(5,4)+qfinal(6,4))**2
     &     -(qfinal(5,1)+qfinal(6,1))**2
     &     -(qfinal(5,2)+qfinal(6,2))**2
     &     -(qfinal(5,3)+qfinal(6,3))**2
        if (jetsfound >= 3) then
        m57=(qfinal(5,4)+qfinal(7,4))**2
     &     -(qfinal(5,1)+qfinal(7,1))**2
     &     -(qfinal(5,2)+qfinal(7,2))**2
     &     -(qfinal(5,3)+qfinal(7,3))**2
        m67=(qfinal(6,4)+qfinal(7,4))**2
     &     -(qfinal(6,1)+qfinal(7,1))**2
     &     -(qfinal(6,2)+qfinal(7,2))**2
     &     -(qfinal(6,3)+qfinal(7,3))**2
        m56=max(m56,max(m57,m67))
        endif
c        if ((m56 < bbsqmin) .or. (m56 > bbsqmax)) then
c          failed=.true.
c        endif
      endif
                  
      return
      end
