!  Copyright (C) 2019-2021, respective authors of MCFM.
!
!  This program is free software: you can redistribute it and/or modify it under
!  the terms of the GNU General Public License as published by the Free Software
!  Foundation, either version 3 of the License, or (at your option) any later
!  version.
!
!  This program is distributed in the hope that it will be useful, but WITHOUT ANY
!  WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS FOR A
!  PARTICULAR PURPOSE. See the GNU General Public License for more details.
!
!  You should have received a copy of the GNU General Public License along with
!  this program. If not, see <http://www.gnu.org/licenses/>
 
      subroutine scaleset_ddis(p,mu0)
        use singletop2_scale_m
      implicit none
      include 'src/Inc/types.f'
c--- subroutine to calculate dynamic scale equal to
c--- "double deep-inelastic scale" defined for the single top t-channel by:
c---        Q^2=-(pt-pb)^2 on the light quark line
c---        Q^2+mt^2 on the heavy quark line
c---  (c.f. Z. Sullivan, hep-ph/0408049)
c--- AVAILABLE AT LEADING ORDER ONLY
      
      include 'src/Inc/constants.f'
      include 'src/Inc/nf.f'
      include 'src/Inc/mxpart.f'
      include 'src/Inc/cplx.h'
      include 'src/Inc/kprocess.f'
      include 'src/Inc/masses.f'
      include 'src/Inc/kpart.f'
      real(dp):: p(mxpart,4),mu0
      include 'src/Inc/bqscale.f'
      real(dp) :: dotvec

      if((kcase==kbq_tpq) .or.
     &   (kcase==kqg_tbq) .or. kcase==ktopanom) then
        if (kcase==kbq_tpq .or. kcase==ktopanom) then

            q1scale = -dotvec(p(1,:)+p(6,:), p(1,:)+p(6,:))
            q2scale = -dotvec(p(2,:)+p(6,:), p(2,:)+p(6,:))
            b1scale = (q2scale + mt**2)
            b2scale = (q1scale + mt**2)

        else
          q1scale=-(
     &         +(p(5,4)+p(1,4))**2
     &         -(p(5,1)+p(1,1))**2
     &         -(p(5,2)+p(1,2))**2
     &         -(p(5,3)+p(1,3))**2)
          b2scale=-(
     &         +(p(5,4)+p(1,4))**2
     &         -(p(5,1)+p(1,1))**2
     &         -(p(5,2)+p(1,2))**2
     &         -(p(5,3)+p(1,3))**2)+mt**2
          q2scale=-(
     &         +(p(5,4)+p(2,4))**2
     &         -(p(5,1)+p(2,1))**2
     &         -(p(5,2)+p(2,2))**2
     &         -(p(5,3)+p(2,3))**2)
          b1scale=-(
     &         +(p(5,4)+p(2,4))**2
     &         -(p(5,1)+p(2,1))**2
     &         -(p(5,2)+p(2,2))**2
     &         -(p(5,3)+p(2,3))**2)+mt**2
      endif
      q1scale=max(sqrt(q1scale),one) ! min. of 1 GeV for safety
      b2scale=max(sqrt(b2scale),one)
      q2scale=max(sqrt(q2scale),one)
      b1scale=max(sqrt(b1scale),one)
      mu0=q1scale   ! for safety
      else
        write(6,*) 'dynamicscale DDIS not supported for this process.'
        stop
      endif
      
c     if (kpart.ne.klord) then
c       write(6,*) 'dynamicscale DDIS only available at LO.'
c       stop      
c     endif
      
      return
      end
      
