!  Copyright (C) 2019-2021, respective authors of MCFM.
!
!  This program is free software: you can redistribute it and/or modify it under
!  the terms of the GNU General Public License as published by the Free Software
!  Foundation, either version 3 of the License, or (at your option) any later
!  version.
!
!  This program is distributed in the hope that it will be useful, but WITHOUT ANY
!  WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS FOR A
!  PARTICULAR PURPOSE. See the GNU General Public License for more details.
!
!  You should have received a copy of the GNU General Public License along with
!  this program. If not, see <http://www.gnu.org/licenses/>
 
      subroutine gen_Vphotons_jets_dkrad(r,nphots,njets,p,wt,*)
      implicit none
      include 'src/Inc/types.f'
!---- generate phase space for 2-->2+nphots+njets process
!----   with (3+4+photon) being a vector boson
!----   and 5,..,4+nphots-1 the other photons (if nphots>1)
!----   and 5+nphots,..,4+nphots+njets the jets
!----
!---- note: as denotd above, the final photon (4+nphots) is generated 
!----       in the decay of the vector boson
!-----
!---- This routine is adapted from gen_njets.f
!----
!---- r(mxdim),p1(4),p2(4) are inputs reversed in sign 
!---- from physical values 
!---- phase space for -p1-p2 --> p3+p4+p5+p6
!---- with all 2 pi's (ie 1/(2*pi)^(4+2n))
!----
      
      include 'src/Inc/constants.f'
      include 'src/Inc/nf.f'
      include 'src/Inc/mxpart.f'
      include 'src/Inc/cplx.h'
      include 'src/Inc/mxdim.f'
      include 'src/Inc/limits.f'
      include 'src/Inc/leptcuts.f'
      include 'src/Inc/reset.f'
      include 'src/Inc/breit.f'
      include 'src/Inc/kpart.f'
      include 'src/Inc/kprocess.f'
      include 'src/Inc/ipsgen.f'
      include 'src/Inc/x1x2.f'
      include 'src/Inc/first.f'
      real(dp):: r(mxdim)
      real(dp):: p(mxpart,4),p3(4),psumjet(4),pcm(4),Q(4)
      real(dp):: wt,dot,s346
      real(dp):: hmin,hmax,delh,h,pt,etamax,etamin
      real(dp):: y,sinhy,coshy,phi,mv2,wtbw,mjets
      real(dp):: ybar,ptsumjet2,ycm,sumpst,q0st,rshat,dely
      real(dp):: ptjetmin,etajetmin,etajetmax,pbreak
      real(dp):: plstar,estar,plstarsq,y5starmax,y5starmin
      real(dp):: p45(4),p4(4),p5(4),wt45,wt345
      integer:: j,nu,nphots,njets,nphotsjets,ijet
      include 'src/Inc/energy.f'
      

c--- total number of photons and jets generated outside of boson decay
      nphotsjets=nphots-1+njets

      do nu=1,4
        do j=1,4+nphotsjets
          p(j,nu)=0._dp
        enddo
        Q(nu)=0._dp 
        psumjet(nu)=0._dp
        pcm(nu)=0._dp
      enddo 

      wt=2._dp*pi
                       
      do ijet=1,nphotsjets
!--- generate the pt of jet number ijet
!--- rapidity limited by E=pT*coshy
        wt=wt/16._dp/pi**3

        pt = r(ijet)**2*sqrts/2._dp
        wt = wt * pt*sqrts/2._dp*2._dp*r(ijet)

        etamax=sqrts/2._dp/pt
        if (etamax**2 <= 1._dp) then
          write(6,*) 'etamax**2 <= 1._dp in gen_phots_jets.f',etamax**2 
          wt=0._dp
          return 1
        endif
        etamax=log(etamax+sqrt(etamax**2-1._dp))
        
        etamax=min(etamax,20d0)
        y=etamax*(2._dp*r(nphotsjets+ijet)-1._dp)
        wt=wt*2._dp*etamax
        
        sinhy=sinh(y)
        coshy=sqrt(1._dp+sinhy**2)
        
        p(5+ijet,4)=pt*coshy
        
        phi=2._dp*pi*r(2*nphotsjets+ijet)
        wt=wt*2._dp*pi
        
        p(5+ijet,1)=pt*cos(phi)
        p(5+ijet,2)=pt*sin(phi)
        p(5+ijet,3)=pt*sinhy
        
        do nu=1,4
          psumjet(nu)=psumjet(nu)+p(5+ijet,nu)
        enddo
      enddo
      
!--- now generate Breit-Wigner        
      call breitw(r(3*nphotsjets+1),wsqmin,sqrts**2,mass3,width3,mv2,wtbw)
      wt=wt*wtbw/2._dp/pi
!--- invariant mass of jets
      mjets=psumjet(4)**2-psumjet(1)**2-psumjet(2)**2-psumjet(3)**2
      mjets=sqrt(abs(mjets))
      
      ybar=0.5_dp*log((psumjet(4)+psumjet(3))/(psumjet(4)-psumjet(3)))
      ptsumjet2=psumjet(1)**2+psumjet(2)**2
      plstarsq=((sqrts**2-mv2-mjets**2)**2
     & -4._dp*(mjets**2*mv2+ptsumjet2*sqrts**2))/(4._dp*sqrts**2)
      if (plstarsq <= 0._dp) then
        wt=0._dp
        return 1
      endif
      plstar=sqrt(plstarsq)
      Estar=sqrt(plstarsq+ptsumjet2+mjets**2)
      y5starmax=0.5_dp*log((Estar+plstar)/(Estar-plstar))
      y5starmin=-y5starmax

      etamax=ybar-y5starmin
      etamin=ybar-y5starmax
      dely=etamax-etamin
      ycm=etamin+r(3*nphotsjets+2)*dely     
      sinhy=sinh(ycm)
      coshy=sqrt(1._dp+sinhy**2)
      
!--- now make the initial state momenta
      sumpst=ptsumjet2+(psumjet(3)*coshy-psumjet(4)*sinhy)**2
      q0st=sqrt(mv2+sumpst)
      rshat=q0st+sqrt(mjets**2+sumpst)
      pcm(4)=rshat*coshy
      pcm(3)=rshat*sinhy
            
      xx(1)=(pcm(4)+pcm(3))/sqrts
      xx(2)=(pcm(4)-pcm(3))/sqrts
      
      if   ((xx(1) > 1._dp) .or. (xx(2) > 1._dp)) then
         wt=0._dp
         return 1
      endif 
      
      wt=wt*dely
      do j=1,4
        Q(j)=pcm(j)-psumjet(j)
      enddo
      
      p(1,4)=-xx(1)*sqrts/2._dp
      p(1,3)=p(1,4)
      p(2,4)=-xx(2)*sqrts/2._dp
      p(2,3)=-p(2,4)
      
      wt=wt*rshat/(sqrts**2*q0st)
      
!--- now decay Q into 3,4,5
      call phi1_2m(zip,r(3*nphotsjets+3),r(3*nphotsjets+4),
     & r(3*nphotsjets+5),zip,Q,p3,p45,wt345,*999)     
      call phi3m0(r(3*nphotsjets+6),r(3*nphotsjets+7),
     & p45,p4,p5,wt45,*999)
      wt=wt*wt345*wt45/twopi     

      do nu=1,4
      if (r(3*nphotsjets+8) < 0.5_dp) then
        p(3,nu)=p3(nu)
        p(4,nu)=p4(nu)
      else
        p(3,nu)=p4(nu)
        p(4,nu)=p3(nu)
      endif
      p(5,nu)=p5(nu)
      enddo
            
      return
      
  999 wt=0._dp
      return 1     
      
      end
      
      
      
      
      
      
      
      
      
      
      
