!  Copyright (C) 2019-2021, respective authors of MCFM.
!
!  This program is free software: you can redistribute it and/or modify it under
!  the terms of the GNU General Public License as published by the Free Software
!  Foundation, either version 3 of the License, or (at your option) any later
!  version.
!
!  This program is distributed in the hope that it will be useful, but WITHOUT ANY
!  WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS FOR A
!  PARTICULAR PURPOSE. See the GNU General Public License for more details.
!
!  You should have received a copy of the GNU General Public License along with
!  this program. If not, see <http://www.gnu.org/licenses/>
 
      subroutine lumxmsq_gaga(p,xx,z1,z2,QB,order,xmsq,central)
          use MCFMPDF
          use SCET
#ifdef HAVE_LHAPDF
          use LHAPDF
#endif
      implicit none
      include 'src/Inc/types.f'
!====== C. Williams July 2015
c----Matrix element for Ga Ga production
C----averaged over initial colours and spins
c===== based upon similar routines for W and Z
      include 'src/Inc/constants.f'
      include 'src/Inc/nf.f'
      include 'src/Inc/mxpart.f'
      include 'src/Inc/masses.f'
      include 'src/Inc/ewcouple.f'
      include 'src/Inc/qcdcouple.f'
      include 'src/Inc/scale.f'
      include 'src/Inc/facscale.f'
      include 'src/Inc/ewcharge.f'
      include 'src/Inc/scet_const.f'
      include 'src/Inc/taucut.f'
      integer:: j,k,ih1,ih2,m,n,order
      real(dp),intent(out):: xmsq
      real(dp):: p(mxpart,4),s,fac,qqb,qbq,
     & xx(2),soft1(-1:1),soft2(-1:3),hard(2),
     & beama0(-5:5),beamb0(-5:5),
     & beama1(-5:5,-1:1),beamb1(-5:5,-1:1),
     & beama2(-5:5,-1:3),beamb2(-5:5,-1:3),
     & z1,z2,QB(2),lum0,lum1(-1:1),lum2(-1:3),bit,
     & msq(-nf:nf,-nf:nf),assemble,Haa(2),getxmsq_gamgam
      common/density/ih1,ih2
      real(dp)::statfac
      parameter(statfac=0.5_dp)
      real(dp):: msqt(-nf:nf,-nf:nf)
      real(dp):: qsum,tlrem,bitrem
      real(dp):: gggaga,msqgggaga,facgg
      
      logical, intent(in) :: central
      real(dp) :: origtaucut

      s(j,k)=two*(p(j,4)*p(k,4)-p(j,1)*p(k,1)
     &           -p(j,2)*p(k,2)-p(j,3)*p(k,3))

      fac=xn*esq**2*statfac
      gggaga=zip
      qsum=zip
      do j=1,nf
         qsum=qsum+Q(j)**2
      enddo
!======= compute Matrix elements 
!      write(6,*) 'old routine'
!      call gamgamampsq(order,p,1,2,3,4,qqb,hard,tlrem)
!      write(6,*) 'hard(1),hard(2)',hard(1),hard(2)
!      write(6,*) 'new routine'
      call gamgamampsq_new(order,p,1,2,3,4,qqb,hard,tlrem)
!      write(6,*) 'hard(1),hard(2)',hard(1),hard(2)
!      pause
      
      qqb=fac*aveqq*qqb
      qbq=qqb

     
      call softqqbis(order,soft1,soft2)

      if (order >= 0) then
      call fdist(ih1,xx(1),facscale,beama0,1)
      call fdist(ih2,xx(2),facscale,beamb0,2)
      endif
      if (order >= 1) then
      call xbeam1bis(ih1,z1,xx(1),QB(1),beama1,1)
      call xbeam1bis(ih2,z2,xx(2),QB(2),beamb1,2)
      endif
      if (order >= 2) then
      call xbeam2bis(ih1,z1,xx(1),QB(1),beama2,1)
      call xbeam2bis(ih2,z2,xx(2),QB(2),beamb2,2)
      facgg=4._dp*esq*gsq/(16._dp*pisq)*Qsum
      gggaga=avegg*V*facgg**2*msqgggaga(s(1,2),s(1,3),s(2,3))*statfac
!      gggaga=zip
      endif

      xmsq=getxmsq_gamgam(p,xx,order,soft1,soft2,hard,
     &              beama0,beamb0,beama1,beamb1,beama2,beamb2,fac,qsum,qqb,tlrem,gggaga)
     
      if (central .and. doMultitaucut) then
          scetreweight(:) = 0._dp
          if (xmsq /= 0._dp) then
              origtaucut = taucut
              do m=1,size(tcutarray)
                  taucut = tcutarray(m)
                  scetreweight(m) = getxmsq_gamgam(p,xx,order,soft1,soft2,hard,
     &              beama0,beamb0,beama1,beamb1,beama2,beamb2,fac,qsum,qqb,tlrem,gggaga)
              enddo
              taucut = origtaucut
              scetreweight(:) = scetreweight(:) / xmsq
          endif
      endif

      return
      end
      
      
      function getxmsq_gamgam(p,xx,order,soft1,soft2,hard,
     &              beama0,beamb0,beama1,beamb1,beama2,beamb2,fac,qsum,qqb,tlrem,gggaga)
          use SCET
      implicit none
      include 'src/Inc/types.f'
      include 'src/Inc/constants.f'
      include 'src/Inc/nf.f'
      include 'src/Inc/mxpart.f'
      include 'src/Inc/masses.f'
      include 'src/Inc/ewcouple.f'
      include 'src/Inc/qcdcouple.f'
      include 'src/Inc/scale.f'
      include 'src/Inc/facscale.f'
      include 'src/Inc/ewcharge.f'
      include 'src/Inc/scet_const.f'
      include 'src/Inc/taucut.f'
      real(dp):: getxmsq_gamgam
      integer:: j,k,m,n,order
      real(dp):: p(mxpart,4),s,fac,qqb,qbq,
     & xx(2),soft1(-1:1),soft2(-1:3),hard(2),
     & beama0(-5:5),beamb0(-5:5),
     & beama1(-5:5,-1:1),beamb1(-5:5,-1:1),
     & beama2(-5:5,-1:3),beamb2(-5:5,-1:3),
     & z1,z2,QB(2),lum0,lum1(-1:1),lum2(-1:3),bit,
     & msq(-nf:nf,-nf:nf),assemble,Haa(2)
      real(dp):: Qh, dot, powc(-5:5,-5:5)
      real(dp)::statfac
      parameter(statfac=0.5_dp)
      real(dp):: msqt(-nf:nf,-nf:nf)
      real(dp):: qsum,tlrem,bitrem
      real(dp):: gggaga,msqgggaga,facgg, tauc

      qbq=qqb

      tauc=getdynamictau(p,taucut)

! compute power corrections if required
! note that, for this process, the power corrections computed here
! are not expected to provide any improvement due to t-channel diagrams
      if ((incpowcorr) .or. (onlypowcorr)) then
        Qh=sqrt(two*dot(p,1,2))
        call powcorr_qa(order,tauc,xx(1),xx(2),Qh,beama0,beamb0,powc)
      endif

      getxmsq_gamgam=zip
      do j=-nf,nf
         k=-j
      
         if (j*k > 0) cycle    ! skip, qq, aa
      
         if (onlypowcorr) then
           bit=zip
           bitrem=zip
         else
           bit=assemble(order,tauc,
     &          beama0(j),beamb0(k),beama1(j,:),beamb1(k,:),
     &          beama2(j,:),beamb2(k,:),soft1,soft2,hard)
           bitrem=fac*aveqq*tlrem*beama0(j)*beamb0(k)*ason2pi**2
         endif
      
         if ((incpowcorr) .or. (onlypowcorr)) then
           bit=bit+powc(j,k)+powc(j,0)+powc(0,k)
         endif

         if ((j > 0) .and. (k < 0)) then
            bit=bit*qqb*Q(j)**4
            if(order > 1) then
!======add on two-loop A functions which go like sum over quark charges
               bit=bit+bitrem*qsum*Q(j)**2
            endif
         elseif ((j < 0) .and. (k > 0)) then
            bit=bit*qbq*Q(k)**4
            if(order > 1) then
!======add on two-loop A functions which go like sum over quark charges
               bit=bit+bitrem*qsum*Q(k)**2
            endif
         elseif ((j==0).and.(k==0).and.(order >= 2 )) then
            if (onlypowcorr) then
              bit=zip
            else
              bit=beama0(j)*beamb0(k)*gggaga
            endif
         else
            bit=zip
         endif
         
         getxmsq_gamgam=getxmsq_gamgam+bit
         
      enddo
      return
      end
