!  Copyright (C) 2019-2021, respective authors of MCFM.
!
!  This program is free software: you can redistribute it and/or modify it under
!  the terms of the GNU General Public License as published by the Free Software
!  Foundation, either version 3 of the License, or (at your option) any later
!  version.
!
!  This program is distributed in the hope that it will be useful, but WITHOUT ANY
!  WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS FOR A
!  PARTICULAR PURPOSE. See the GNU General Public License for more details.
!
!  You should have received a copy of the GNU General Public License along with
!  this program. If not, see <http://www.gnu.org/licenses/>
 
      function amp_2gam2g_mppppm(j1,j2,j3,j4,j5,j6,za,zb)
      implicit none
      include 'src/Inc/types.f'
      complex(dp):: amp_2gam2g_mppppm
c--- -i*Amplitude for the process
c---       0  -->  qb(j5) + q(j6) + g(j1) + g(j2) + gam(j3) + gam(j4)
c--- helicities:     +       -      -       +         +         +
c---
c--- Taken from "Multi-Photon Amplitudes for Next-to-Leading Order QCD"
c---  V. Del Duca, W. Kilgore and F. Maltoni, hep-ph/9910253
c---
      
      include 'src/Inc/constants.f'
      include 'src/Inc/nf.f'
      include 'src/Inc/mxpart.f'
      include 'src/Inc/cplx.h'
      include 'src/Inc/sprods_com.f'
      include 'src/Inc/zprods_decl.f'
      integer:: j1,j2,j3,j4,j5,j6

c--- Equation (2.9) with r=2, m=2 and labels permuted appropriately
c--- negative helicity gluon i=1
      amp_2gam2g_mppppm=      
     & za(j5,j1)*za(j6,j1)**3/(za(j5,j6)*za(j6,j1)*za(j1,j2)*za(j2,j5))
     & *za(j6,j5)/(za(j6,j3)*za(j3,j5))
     & *za(j6,j5)/(za(j6,j4)*za(j4,j5))
     
      return
      end

      
      function amp_2gam2g_pmpppm(j1,j2,j3,j4,j5,j6,za,zb)
      implicit none
      include 'src/Inc/types.f'
      complex(dp):: amp_2gam2g_pmpppm
c--- -i*Amplitude for the process
c---       0  -->  qb(j5) + q(j6) + g(j1) + g(j2) + gam(j3) + gam(j4)
c--- helicities:     +       -      +       -         +         +
c---
c--- Taken from "Multi-Photon Amplitudes for Next-to-Leading Order QCD"
c---  V. Del Duca, W. Kilgore and F. Maltoni, hep-ph/9910253
c---
      
      include 'src/Inc/constants.f'
      include 'src/Inc/nf.f'
      include 'src/Inc/mxpart.f'
      include 'src/Inc/cplx.h'
      include 'src/Inc/sprods_com.f'
      include 'src/Inc/zprods_decl.f'
      integer:: j1,j2,j3,j4,j5,j6

c--- Equation (2.9) with r=2, m=2 and labels permuted appropriately
c--- negative helicity gluon i=2
      amp_2gam2g_pmpppm=      
     & za(j5,j2)*za(j6,j2)**3/(za(j5,j6)*za(j6,j1)*za(j1,j2)*za(j2,j5))
     & *za(j6,j5)/(za(j6,j3)*za(j3,j5))
     & *za(j6,j5)/(za(j6,j4)*za(j4,j5))
     
      return
      end

      
      function amp_2gam2g_ppmppm(j1,j2,j3,j4,j5,j6,za,zb)
      implicit none
      include 'src/Inc/types.f'
      complex(dp):: amp_2gam2g_ppmppm
c--- -i*Amplitude for the process
c---       0  -->  qb(j5) + q(j6) + g(j1) + g(j2) + gam(j3) + gam(j4)
c--- helicities:     +       -      +       +         -         +
c---
c--- Taken from "Multi-Photon Amplitudes for Next-to-Leading Order QCD"
c---  V. Del Duca, W. Kilgore and F. Maltoni, hep-ph/9910253
c---
      
      include 'src/Inc/constants.f'
      include 'src/Inc/nf.f'
      include 'src/Inc/mxpart.f'
      include 'src/Inc/cplx.h'
      include 'src/Inc/sprods_com.f'
      include 'src/Inc/zprods_decl.f'
      integer:: j1,j2,j3,j4,j5,j6

c--- Equation (2.9) with r=2, m=2 and labels permuted appropriately
c--- negative helicity photon i=3
      amp_2gam2g_ppmppm=      
     & za(j5,j3)*za(j6,j3)**3/(za(j5,j6)*za(j6,j1)*za(j1,j2)*za(j2,j5))
     & *za(j6,j5)/(za(j6,j3)*za(j3,j5))
     & *za(j6,j5)/(za(j6,j4)*za(j4,j5))
     
      return
      end

      
      function amp_2gam2g_pppmpm(j1,j2,j3,j4,j5,j6,za,zb)
      implicit none
      include 'src/Inc/types.f'
      complex(dp):: amp_2gam2g_pppmpm
c--- -i*Amplitude for the process
c---       0  -->  qb(j5) + q(j6) + g(j1) + g(j2) + gam(j3) + gam(j4)
c--- helicities:     +       -      +       +         +         -
c---
c--- Taken from "Multi-Photon Amplitudes for Next-to-Leading Order QCD"
c---  V. Del Duca, W. Kilgore and F. Maltoni, hep-ph/9910253
c---
      
      include 'src/Inc/constants.f'
      include 'src/Inc/nf.f'
      include 'src/Inc/mxpart.f'
      include 'src/Inc/cplx.h'
      include 'src/Inc/sprods_com.f'
      include 'src/Inc/zprods_decl.f'
      integer:: j1,j2,j3,j4,j5,j6

c--- Equation (2.9) with r=2, m=2 and labels permuted appropriately
c--- negative helicity photon i=4
      amp_2gam2g_pppmpm=      
     & za(j5,j4)*za(j6,j4)**3/(za(j5,j6)*za(j6,j1)*za(j1,j2)*za(j2,j5))
     & *za(j6,j5)/(za(j6,j3)*za(j3,j5))
     & *za(j6,j5)/(za(j6,j4)*za(j4,j5))
     
      return
      end

      
      function amp_2gam2g_mmpppm(j1,j2,j3,j4,j5,j6,za,zb)
      implicit none
      include 'src/Inc/types.f'
      complex(dp):: amp_2gam2g_mmpppm
c--- -i*Amplitude for the process
c---       0  -->  qb(j5) + q(j6) + g(j1) + g(j2) + gam(j3) + gam(j4)
c--- helicities:     +       -      -       -         +         +
c---
c--- Taken from "Multi-Photon Amplitudes for Next-to-Leading Order QCD"
c---  V. Del Duca, W. Kilgore and F. Maltoni, hep-ph/9910253
c---
      
      include 'src/Inc/constants.f'
      include 'src/Inc/nf.f'
      include 'src/Inc/mxpart.f'
      include 'src/Inc/cplx.h'
      include 'src/Inc/sprods_com.f'
      include 'src/Inc/zprods_decl.f'
      integer:: j1,j2,j3,j4,j5,j6
      real(dp):: t
      
c--- Equation (4.1)
c--- note: I have interpreted, e.g. <5|1+2|6> as [5|1+2|6> in order
c--- to get the correct <| and |] required by the helicities
      amp_2gam2g_mmpppm=      
     &+(zb(j5,j1)*za(j1,j6)+zb(j5,j2)*za(j2,j6))*za(j5,j6)*t(j1,j2,j6)
     & /(zb(j5,j2)*zb(j1,j6)*zb(j2,j1)*za(j6,j3)
     &  *za(j6,j4)*za(j5,j3)*za(j5,j4))
     &-za(j1,j6)*zb(j4,j5)*(zb(j3,j1)*za(j1,j2)+zb(j3,j6)*za(j6,j2))
     & /(zb(j1,j6)*za(j4,j5)*zb(j5,j2)*za(j6,j3)*t(j1,j3,j6))
     &-za(j1,j6)*zb(j3,j5)*(zb(j4,j1)*za(j1,j2)+zb(j4,j6)*za(j6,j2))
     & /(zb(j1,j6)*za(j3,j5)*zb(j5,j2)*za(j6,j4)*t(j1,j4,j6))
      return
      end

      
      function amp_2gam2g_pmmppm(j1,j2,j3,j4,j5,j6,za,zb)
      implicit none
      include 'src/Inc/types.f'
      complex(dp):: amp_2gam2g_pmmppm
c--- -i*Amplitude for the process
c---       0  -->  qb(j5) + q(j6) + g(j1) + g(j2) + gam(j3) + gam(j4)
c--- helicities:     +       -      +       -         -         +
c---
c--- Taken from "Multi-Photon Amplitudes for Next-to-Leading Order QCD"
c---  V. Del Duca, W. Kilgore and F. Maltoni, hep-ph/9910253
c---
      
      include 'src/Inc/constants.f'
      include 'src/Inc/nf.f'
      include 'src/Inc/mxpart.f'
      include 'src/Inc/cplx.h'
      include 'src/Inc/sprods_com.f'
      include 'src/Inc/zprods_decl.f'
      integer:: j1,j2,j3,j4,j5,j6
      real(dp):: t
      
c--- Equation (4.2)
c--- note: I have interpreted, e.g. <4|3+6|2> as [4|3+6|2> in order
c--- to get the correct <| and |] required by the helicities
      amp_2gam2g_pmmppm=      
     &+zb(j4,j5)*za(j2,j6)**2*(zb(j1,j2)*za(j2,j3)+zb(j1,j6)*za(j6,j3))
     & /(za(j4,j5)*zb(j5,j3)*za(j1,j6)*s(j1,j2)*t(j1,j2,j6))
     &+za(j3,j6)*zb(j5,j4)*(zb(j1,j3)*za(j3,j2)+zb(j1,j6)*za(j6,j2))
     & /(zb(j3,j6)*za(j5,j4)*zb(j5,j2)*za(j1,j6)*t(j1,j3,j6))
     &+za(j3,j6)*zb(j1,j5)**2*(zb(j4,j3)*za(j3,j2)+zb(j4,j6)*za(j6,j2))
     & /(zb(j3,j6)*zb(j2,j5)*za(j6,j4)*s(j1,j2)*t(j1,j2,j5))
     &+zb(j5,j1)*za(j6,j2)*(zb(j5,j1)*za(j1,j6)+zb(j5,j4)*za(j4,j6))
     & *t(j1,j4,j5)/(zb(j3,j6)*zb(j5,j2)*zb(j5,j3)
     &              *za(j6,j1)*za(j6,j4)*za(j4,j5)*s(j1,j2))
     &+(zb(j1,j3)*za(j3,j2)+zb(j1,j6)*za(j6,j2))
     & *(zb(j5,j1)*za(j1,j6)*(zb(j5,j2)*za(j2,j6)+zb(j5,j3)*za(j3,j6))
     &  +zb(j5,j4)*za(j4,j6)*zb(j5,j2)*za(j2,j6))
     & /(zb(j3,j6)*zb(j5,j2)*zb(j5,j3)*za(j6,j1)
     &  *za(j6,j4)*za(j4,j5)*s(j1,j2))

      return
      end
    
      
      function amp_2gam2g_mpmppm(j1,j2,j3,j4,j5,j6,za,zb)
      implicit none
      include 'src/Inc/types.f'
      complex(dp):: amp_2gam2g_mpmppm
c--- -i*Amplitude for the process
c---       0  -->  qb(j5) + q(j6) + g(j1) + g(j2) + gam(j3) + gam(j4)
c--- helicities:     +       -      -       +         -         +
c---
c--- Taken from "Multi-Photon Amplitudes for Next-to-Leading Order QCD"
c---  V. Del Duca, W. Kilgore and F. Maltoni, hep-ph/9910253
c---
      
      include 'src/Inc/constants.f'
      include 'src/Inc/nf.f'
      include 'src/Inc/mxpart.f'
      include 'src/Inc/cplx.h'
      include 'src/Inc/sprods_com.f'
      include 'src/Inc/zprods_decl.f'
      integer:: j1,j2,j3,j4,j5,j6
      real(dp):: t
      
c--- Equation (4.3)
c--- note: I have interpreted, e.g. <4|2+5|3> as [4|2+5|3> in order
c--- to get the correct <| and |] required by the helicities
      amp_2gam2g_mpmppm=      
     &+za(j1,j6)*zb(j2,j5)*(zb(j4,j2)*za(j2,j3)+zb(j4,j5)*za(j5,j3))
     & /(zb(j1,j6)*za(j2,j5)*zb(j5,j3)*za(j6,j4)*t(j1,j4,j6))
     &+za(j1,j6)*zb(j4,j5)*zb(j6,j2)
     & *(zb(j2,j4)*za(j4,j3)+zb(j2,j5)*za(j5,j3))
     & /(zb(j1,j6)*za(j4,j5)*zb(j5,j3)*s(j1,j2)*t(j1,j2,j6))
     &+za(j3,j6)*zb(j2,j5)*za(j5,j1)
     & *(zb(j4,j2)*za(j2,j1)+zb(j4,j5)*za(j5,j1))
     & /(zb(j3,j6)*za(j2,j5)*za(j6,j4)*s(j1,j2)*t(j3,j4,j6))
     &+za(j1,j6)*zb(j2,j5)*(zb(j2,j4)*za(j4,j3)+zb(j2,j5)*za(j5,j3))
     & /(zb(j1,j6)*zb(j3,j5)*za(j5,j4)*za(j4,j6)*s(j1,j2))
     &+t(j1,j3,j6)*(za(j5,j1)*zb(j2,j6)*zb(j5,j3)*za(j3,j6)
     &             +zb(j1,j2)*zb(j6,j5)*za(j1,j6)*za(j5,j1))
     & /(zb(j1,j6)*zb(j5,j3)*zb(j3,j6)*za(j2,j5)
     &  *za(j5,j4)*za(j4,j6)*s(j1,j2))

      return
      end
      
