!  Copyright (C) 2019-2021, respective authors of MCFM.
!
!  This program is free software: you can redistribute it and/or modify it under
!  the terms of the GNU General Public License as published by the Free Software
!  Foundation, either version 3 of the License, or (at your option) any later
!  version.
!
!  This program is distributed in the hope that it will be useful, but WITHOUT ANY
!  WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS FOR A
!  PARTICULAR PURPOSE. See the GNU General Public License for more details.
!
!  You should have received a copy of the GNU General Public License along with
!  this program. If not, see <http://www.gnu.org/licenses/>
 
      subroutine xwqqgg_v(mqqb)
      implicit none
      include 'src/Inc/types.f'
      
************************************************************************
*     Author J.M.Campbell, February 2000                               *
*     Returns the interference of the tree and loop                    *
*     amplitudes for the process                                       *
*     0---> q(p1)+g(p2)+g(p3)+qbar(p4)+l(p5)+a(p6)                     *
*                                                                      *
*     The value of COLOURCHOICE determines which colour structures     *
*     are included in the calculation                                  *
************************************************************************

      include 'src/Inc/constants.f'
      include 'src/Inc/nf.f'
      include 'src/Inc/mxpart.f'
      include 'src/Inc/zprods_com.f'
      include 'src/Inc/ewcouple.f'
      include 'src/Inc/qcdcouple.f'
      include 'src/Inc/lc.f'
      include 'src/Inc/heldefs.f'
      integer:: j,lh,h2,h3,hq,h(2:3)
      real(dp):: mqqb,fac
      complex(dp):: m(2),ml1(2),ml2(2),ml3,ml4(2)
      complex(dp):: a6treeg1,a61g1lc,a61g1slc,a61g1nf,a63g1
      integer,parameter:: 
     & i1(2)=(/1,4/),i2(2)=(/2,3/),i3(2)=(/3,2/),
     & i4(2)=(/4,1/),i5(2)=(/6,5/),i6(2)=(/5,6/)
      integer,parameter:: st1(2,2)=reshape(
     & (/hqpgmgmqbm ,hqpgmgpqbm ,hqpgpgmqbm ,hqpgpgpqbm /),(/2,2/))
      integer,parameter:: st2(2,2)=reshape(
     & (/hqpqbmgpgp ,hqpqbmgpgm ,hqpqbmgmgp ,hqpqbmgmgm /),(/2,2/))
      integer,parameter:: st3(2,2)=reshape(
     & (/hqpqbmgmgm ,hqpqbmgmgp ,hqpqbmgpgm ,hqpqbmgpgp /),(/2,2/))
      include 'src/Inc/cplx.h'
      
      
C ---final matrix element squared is needed for left-handed
c--  quark and lepton line helicity

      fac=avegg*8._dp*gsq**2*esq**2*cf*xn**3*ason2pi
c--- no extra factor here since colour algebra is already done in (2.12)

      hq=1
      lh=1
      mqqb=0._dp

      do h2=1,2
      do h3=1,2
        h(2)=h2
        h(3)=h3
        do j=1,2
        if (hq == 1) then        
        m(j)=  a6treeg1(st1(3-h(i2(j)),3-h(i3(j))),
     &     i1(1),i2(j),i3(j),i4(1),i6(lh),i5(lh),zb,za)
        if (colourchoice <= 2) then
        ml1(j)= a61g1lc(st1(3-h(i2(j)),3-h(i3(j))),
     &     i1(1),i2(j),i3(j),i4(1),i6(lh),i5(lh),zb,za)
        endif
        if ((colourchoice >= 2) .or. (colourchoice == 0)) then
        ml2(j)=a61g1slc(st2(3-h(i2(j)),3-h(i3(j))),
     &     i1(1),i2(j),i3(j),i4(1),i6(lh),i5(lh),zb,za)
        endif
        if (colourchoice == 0) then
        ml4(j)=a61g1nf(st1(3-h(i2(j)),3-h(i3(j))),
     &     i1(1),i2(j),i3(j),i4(1),i6(lh),i5(lh),zb,za)
        endif
        else
        m(j)=  a6treeg1(st1(h(i2(j)),h(i3(j))),
     &     i1(1),i2(j),i3(j),i4(1),i5(lh),i6(lh),za,zb)
        if (colourchoice <= 2) then
        ml1(j)= a61g1lc(st1(h(i2(j)),h(i3(j))),
     &     i1(1),i2(j),i3(j),i4(1),i5(lh),i6(lh),za,zb)
        endif
        if ((colourchoice >= 2) .or. (colourchoice == 0)) then
        ml2(j)=a61g1slc(st2(h(i2(j)),h(i3(j))),
     &     i1(1),i2(j),i3(j),i4(1),i5(lh),i6(lh),za,zb)
        endif
        if (colourchoice == 0) then
        ml4(j)=a61g1nf(st1(h(i2(j)),h(i3(j))),
     &     i1(1),i2(j),i3(j),i4(1),i5(lh),i6(lh),za,zb)
        endif
        endif
        enddo

        if ((colourchoice == 2) .or. (colourchoice == 0)) then
        if (hq == 1) then
        ml3=a63g1(st3(3-h2,3-h3),1,4,2,3,i6(lh),i5(lh),zb,za)
        else
        ml3=a63g1(st3(h2,h3),1,4,2,3,i5(lh),i6(lh),za,zb)
        endif
        endif       

      if     (colourchoice == 1) then
        mqqb=mqqb+fac*(real(conjg(m(1))*ml1(1),dp)
     &                +real(conjg(m(2))*ml1(2),dp))
      elseif (colourchoice == 2) then
        mqqb=mqqb+fac*(
     &    real(conjg(m(1))*(
     &     -(ml1(1)+ml2(1)+ml1(2)-ml3)/xnsq
     &     -(ml2(1)+ml2(2))/xnsq),dp)
     &   +real(conjg(m(2))*(
     &     -(ml1(2)+ml2(2)+ml1(1)-ml3)/xnsq
     &     -(ml2(1)+ml2(2))/xnsq),dp))
      elseif (colourchoice == 3) then
        mqqb=mqqb+fac*(1._dp+xnsq)/xnsq**2*
     &                (real(conjg(m(1))*(ml2(1)+ml2(2)),dp)
     &                +real(conjg(m(2))*(ml2(1)+ml2(2)),dp))
      else
        mqqb=mqqb+fac*(
     &    real(conjg(m(1))*(
     &      ml1(1)
     &     -(ml1(1)+ml2(1)+ml1(2)-ml3)/xnsq
     &     +(ml2(1)+ml2(2))/xnsq**2
     &     +ml4(1)/xn
     &     -(ml4(1)+ml4(2))/xn**3),dp)
     &   +real(conjg(m(2))*(
     &      ml1(2)
     &     -(ml1(2)+ml2(2)+ml1(1)-ml3)/xnsq
     &     +(ml2(1)+ml2(2))/xnsq**2
     &     +ml4(2)/xn
     &     -(ml4(1)+ml4(2))/xn**3),dp))
      endif
     
      enddo
      enddo

         
      return
      end
      
