!  Copyright (C) 2019-2021, respective authors of MCFM.
!
!  This program is free software: you can redistribute it and/or modify it under
!  the terms of the GNU General Public License as published by the Free Software
!  Foundation, either version 3 of the License, or (at your option) any later
!  version.
!
!  This program is distributed in the hope that it will be useful, but WITHOUT ANY
!  WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS FOR A
!  PARTICULAR PURPOSE. See the GNU General Public License for more details.
!
!  You should have received a copy of the GNU General Public License along with
!  this program. If not, see <http://www.gnu.org/licenses/>
 
      subroutine qqb_ZH1jet_v(p,msq)
      implicit none
      include 'src/Inc/types.f'
      
************************************************************************
*     Authors: R.K. Ellis and John Campbell                            *
*     May, 2001.                                                       *
*     Matrix element for Z + jet production                            *
*     in order alpha_s^2                                               *
*     averaged over initial colours and spins                          *
*     q(-p1)+qbar(-p2)-->Z^+(l(p3)+a(p4))+H(p5,p6)+g(p7)               *
************************************************************************
!
! Note that some Yukawa-type contributions are included, and axial-anomaly diagrams,
! but we neglect box contributions with external legs (g^* -> q q~, g, Z, H) that decouple
!
      include 'src/Inc/constants.f'
      include 'src/Inc/nf.f'
      include 'src/Inc/mxpart.f'
      include 'src/Inc/masses.f'
      include 'src/Inc/qcdcouple.f'
      include 'src/Inc/ewcharge.f'
      include 'src/Inc/ewcouple.f'
      include 'src/Inc/hdecaymode.f'
      include 'src/Inc/zcouple.f'
      include 'src/Inc/sprods_com.f'
      include 'src/Inc/zprods_com.f'
      include 'src/Inc/epinv.f'
      include 'src/Inc/scheme.f'
      include 'src/Inc/nflav.f'
!      include 'src/Inc/cutoff.f'
      include 'src/Inc/hbbparams.f'
      include 'src/Inc/noglue.f'
      include 'src/Inc/zcouple_cms.f'
      include 'src/Inc/blha.f'
      integer:: j,k,ig
      real(dp):: msq(-nf:nf,-nf:nf),msq0(-nf:nf,-nf:nf),
     & p(mxpart,4),fac,s34,s56,s127,virt5_VH,subuv,
     & msqhtautau,msqhbb,msqhgamgam,hdecay
      complex(dp):: virt5_ZHtop
      real(dp):: qqbZgLL,qqbZgRR,qqbZgLR,qqbZgRL
      real(dp):: gqZqLL,gqZqRR,gqZqLR,gqZqRL
      real(dp):: qgZqLL,qgZqRR,qgZqLR,qgZqRL
      real(dp):: qbqZgLL,qbqZgRR,qbqZgLR,qbqZgRL
      real(dp):: gqbZqbLL,gqbZqbRR,gqbZqbLR,gqbZqbRL
      real(dp):: qbgZqbLL,qbgZqbRR,qbgZqbLR,qbgZqbRL
      complex(dp):: qqbZgLLtop,qqbZgRRtop,qqbZgLRtop,qqbZgRLtop
      complex(dp):: gqZqLLtop,gqZqRRtop,gqZqLRtop,gqZqRLtop
      complex(dp):: qgZqLLtop,qgZqRRtop,qgZqLRtop,qgZqRLtop
      complex(dp):: qbqZgLLtop,qbqZgRRtop,qbqZgLRtop,qbqZgRLtop
      complex(dp):: gqbZqbLLtop,gqbZqbRRtop,gqbZqbLRtop,gqbZqbRLtop
      complex(dp):: qbgZqbLLtop,qbgZqbRRtop,qbgZqbLRtop,qbgZqbRLtop
      complex(dp):: prop,proptop,virt5ax,zsin2winv
      complex(dp):: qqbZgLLax,qqbZgRRax,qqbZgLRax,qqbZgRLax
      complex(dp):: qbqZgLLax,qbqZgRRax,qbqZgLRax,qbqZgRLax
      complex(dp):: qgZqLLax,qgZqRRax,qgZqLRax,qgZqRLax
      complex(dp):: gqbZqbLLax,gqbZqbRRax,gqbZqbLRax,gqbZqbRLax
      complex(dp):: gqZqLLax,gqZqRRax,gqZqLRax,gqZqRLax
      complex(dp):: qbgZqbLLax,qbgZqbRRax,qbgZqbLRax,qbgZqbRLax
      real(dp):: fac_top,sH!,cutoff_orig
      integer,parameter::
     & iqqbgLL(5)=(/1,2,3,4,7/),iqqbgRR(5)=(/2,1,4,3,7/),
     & iqqbgRL(5)=(/2,1,3,4,7/),iqqbgLR(5)=(/1,2,4,3,7/),
     & iqgqLL(5)=(/1,7,3,4,2/),iqgqRR(5)=(/7,1,4,3,2/),
     & iqgqRL(5)=(/7,1,3,4,2/),iqgqLR(5)=(/1,7,4,3,2/),
     & igqqLL(5)=(/2,7,3,4,1/),igqqRR(5)=(/7,2,4,3,1/),
     & igqqRL(5)=(/7,2,3,4,1/),igqqLR(5)=(/2,7,4,3,1/)
      integer,parameter::
     & iqqbgLL9(5)=(/1,2,3,4,9/),iqqbgRR9(5)=(/2,1,4,3,9/),
     & iqqbgRL9(5)=(/2,1,3,4,9/),iqqbgLR9(5)=(/1,2,4,3,9/),
     & iqgqLL9(5)=(/1,9,3,4,2/),iqgqRR9(5)=(/9,1,4,3,2/),
     & iqgqRL9(5)=(/9,1,3,4,2/),iqgqLR9(5)=(/1,9,4,3,2/),
     & igqqLL9(5)=(/2,9,3,4,1/),igqqRR9(5)=(/9,2,4,3,1/),
     & igqqRL9(5)=(/9,2,3,4,1/),igqqLR9(5)=(/2,9,4,3,1/)
      include 'src/Inc/cplx.h'
      common/virt5ax/virt5ax
!$omp threadprivate(/virt5ax/)

      scheme='dred'
c--set msq=0 to initialize
      msq(:,:)=zip

!      cutoff_orig=cutoff

      if(hdecaymode.ne.'wpwm') then
         ig=7
      else
         ig=9
      endif
      
c--calculate spinor and dot-products (using BDK type notation)
      call spinoru(ig,p,za,zb)

c--- calculate lowest order
      msq0=zip
      if (toponly .eqv. .false.) then
        call qqb_ZH1jet(p,msq0)
      endif
      
c----UV counterterm contains the finite renormalization to arrive
c----at MS bar scheme.      
      subuv=ason2pi*xn*(epinv*(11._dp-two*real(nflav,dp)/xn)-one)/6._dp

c--   calculate propagator
      s34=s(3,4)
      s127=s(1,2)+s(1,ig)+s(2,ig)
      proptop=s34/cplx2((s34-zmass**2),zmass*zwidth)
      prop=proptop*s127/cplx2((s127-zmass**2),zmass*zwidth)

C     Deal with Higgs decay
      if (hdecaymode == 'tlta') then
         sH=s(5,6)+two*mtau**2
         hdecay=msqhtautau(sH)
      elseif (hdecaymode == 'bqba') then
         sH=s(5,6)+two*mb**2
         hdecay=msqhbb(sH)
c---  adjust for fixed H->bb BR if necessary
         if (FixBrHbb) then
            hdecay=hdecay*GamHbb/GamHbb0
         endif
      elseif (hdecaymode == 'gaga') then
         sH=s(5,6)
         hdecay=msqhgamgam(sH)
      elseif (hdecaymode == 'wpwm') then
         sH=s(5,6)+s(5,7)+s(5,8)+s(6,7)+s(6,8)+s(7,8)
         call hwwdecay(p,5,6,7,8,hdecay)
      elseif (hdecaymode == 'none') then
         sH=s(5,5)
         hdecay=one
      else
         write(6,*) 'Unimplemented process in qqb_higgs'
         stop
      endif
      if (hdecaymode /= 'none') then
         hdecay=hdecay/((sH-hmass**2)**2+(hmass*hwidth)**2)
      endif
      
      fac=8._dp*cf*xnsq*esq**2*gsq*hdecay
c-- note: in CMS, wmass**2 -> sqrt(wmass**4+(wmass*wwidth)**2)
      fac=fac*gwsq*sqrt(wmass**4+(wmass*wwidth)**2)/abs(cone-zxw)**2
      fac_top=two*cf*xn*hdecay*2._dp

!---- smalls cut for top loop piece stability (quick but dirty)
!      cutoff=1.E-3_dp
      call smalls(s,ig-2,*101)
      goto 102
 101  fac_top=zip
 102  continue 
!      cutoff=cutoff_orig

      if(ig==7) then 
      if ((useblha.eq.0).or.((blhafl(1).ne.0).and.(blhafl(2).ne.0))) then
      qqbZgLLtop=aveqq*fac_top*virt5_ZHtop(iqqbgLL,za,zb)
      qqbZgLRtop=aveqq*fac_top*virt5_ZHtop(iqqbgLR,za,zb)
      qqbZgRLtop=aveqq*fac_top*virt5_ZHtop(iqqbgRL,za,zb)
      qqbZgRRtop=aveqq*fac_top*virt5_ZHtop(iqqbgRR,za,zb)

      if (toponly .eqv. .false.) then
        qqbZgLL=aveqq*fac*virt5_VH(iqqbgLL,za,zb)/s34**2
        qqbZgLLax=-aveqq*fac*virt5ax/s34**2
        qqbZgLR=aveqq*fac*virt5_VH(iqqbgLR,za,zb)/s34**2
        qqbZgLRax=-aveqq*fac*virt5ax/s34**2
        qqbZgRL=aveqq*fac*virt5_VH(iqqbgRL,za,zb)/s34**2
        qqbZgRLax=aveqq*fac*virt5ax/s34**2
        qqbZgRR=aveqq*fac*virt5_VH(iqqbgRR,za,zb)/s34**2
        qqbZgRRax=aveqq*fac*virt5ax/s34**2
      else
        qqbZgLL=0._dp
        qqbZgLR=0._dp
        qqbZgRL=0._dp
        qqbZgRR=0._dp
      endif

      qbqZgLL=qqbZgRL
      qbqZgLR=qqbZgRR
      qbqZgRL=qqbZgLL
      qbqZgRR=qqbZgLR

      qbqZgLLax=-qqbZgRLax
      qbqZgLRax=-qqbZgRRax
      qbqZgRLax=-qqbZgLLax
      qbqZgRRax=-qqbZgLRax

      qbqZgLLtop=qqbZgRLtop
      qbqZgLRtop=qqbZgRRtop
      qbqZgRLtop=qqbZgLLtop
      qbqZgRRtop=qqbZgLRtop
      endif

      if ((useblha.eq.0).or.((blhafl(1).eq.0).and.(blhafl(2).ne.0))) then
      gqZqLLtop=aveqg*fac_top*virt5_ZHtop(igqqLL,za,zb)
      gqZqLRtop=aveqg*fac_top*virt5_ZHtop(igqqLR,za,zb)
      gqZqRLtop=aveqg*fac_top*virt5_ZHtop(igqqRL,za,zb)
      gqZqRRtop=aveqg*fac_top*virt5_ZHtop(igqqRR,za,zb)
    
      if (toponly .eqv. .false.) then
        gqZqLL=aveqg*fac*virt5_VH(igqqLL,za,zb)/s34**2
        gqZqLLax=-aveqg*fac*virt5ax/s34**2
        gqZqLR=aveqg*fac*virt5_VH(igqqLR,za,zb)/s34**2
        gqZqLRax=-aveqg*fac*virt5ax/s34**2
        gqZqRL=aveqg*fac*virt5_VH(igqqRL,za,zb)/s34**2
        gqZqRLax=aveqg*fac*virt5ax/s34**2
        gqZqRR=aveqg*fac*virt5_VH(igqqRR,za,zb)/s34**2
        gqZqRRax=aveqg*fac*virt5ax/s34**2
      else
        gqZqLL=0._dp
        gqZqLR=0._dp
        gqZqRL=0._dp
        gqZqRR=0._dp
      endif

      gqbZqbRL=gqZqLL
      gqbZqbRR=gqZqLR
      gqbZqbLL=gqZqRL
      gqbZqbLR=gqZqRR

      gqbZqbRLax=-gqZqLLax
      gqbZqbRRax=-gqZqLRax
      gqbZqbLLax=-gqZqRLax
      gqbZqbLRax=-gqZqRRax

      gqbZqbRLtop=gqZqLLtop
      gqbZqbRRtop=gqZqLRtop
      gqbZqbLLtop=gqZqRLtop
      gqbZqbLRtop=gqZqRRtop
      endif

      if ((useblha.eq.0).or.((blhafl(1).ne.0).and.(blhafl(2).eq.0))) then
      qgZqLLtop=aveqg*fac_top*virt5_ZHtop(iqgqLL,za,zb)
      qgZqLRtop=aveqg*fac_top*virt5_ZHtop(iqgqLR,za,zb)
      qgZqRLtop=aveqg*fac_top*virt5_ZHtop(iqgqRL,za,zb)
      qgZqRRtop=aveqg*fac_top*virt5_ZHtop(iqgqRR,za,zb)
     
      if (toponly .eqv. .false.) then
        qgZqLL=aveqg*fac*virt5_VH(iqgqLL,za,zb)/s34**2
        qgZqLLax=-aveqg*fac*virt5ax/s34**2
        qgZqLR=aveqg*fac*virt5_VH(iqgqLR,za,zb)/s34**2
        qgZqLRax=-aveqg*fac*virt5ax/s34**2
        qgZqRL=aveqg*fac*virt5_VH(iqgqRL,za,zb)/s34**2
        qgZqRLax=aveqg*fac*virt5ax/s34**2
        qgZqRR=aveqg*fac*virt5_VH(iqgqRR,za,zb)/s34**2
        qgZqRRax=aveqg*fac*virt5ax/s34**2
      else
        qgZqLL=0._dp
        qgZqLR=0._dp
        qgZqRL=0._dp
        qgZqRR=0._dp
      endif

      qbgZqbRL=qgZqLL
      qbgZqbRR=qgZqLR
      qbgZqbLL=qgZqRL
      qbgZqbLR=qgZqRR

      qbgZqbRLax=-qgZqLLax
      qbgZqbRRax=-qgZqLRax
      qbgZqbLLax=-qgZqRLax
      qbgZqbLRax=-qgZqRRax

      qbgZqbRLtop=qgZqLLtop
      qbgZqbRRtop=qgZqLRtop
      qbgZqbLLtop=qgZqRLtop
      qbgZqbLRtop=qgZqRRtop
      endif

      elseif(ig==9) then 
!!! Still need to fix the ig==9 case !!!
         write(6,*) 'Amplitudes for this case need to be improved!'
         stop

         qqbZgLL=aveqq*fac_top*virt5_ZHtop(iqqbgLL9,za,zb)
         qqbZgLR=aveqq*fac_top*virt5_ZHtop(iqqbgLR9,za,zb)
         qqbZgRL=aveqq*fac_top*virt5_ZHtop(iqqbgRL9,za,zb)
         qqbZgRR=aveqq*fac_top*virt5_ZHtop(iqqbgRR9,za,zb)
         
         if (toponly .eqv. .false.) then
            qqbZgLL=qqbZgLL+aveqq*fac*virt5_VH(iqqbgLL9,za,zb)
            qqbZgLR=qqbZgLR+aveqq*fac*virt5_VH(iqqbgLR9,za,zb)
            qqbZgRL=qqbZgRL+aveqq*fac*virt5_VH(iqqbgRL9,za,zb)
            qqbZgRR=qqbZgRR+aveqq*fac*virt5_VH(iqqbgRR9,za,zb)
         endif
         
         qbqZgLL=qqbZgRL
         qbqZgLR=qqbZgRR
         qbqZgRL=qqbZgLL
         qbqZgRR=qqbZgLR

         gqZqLL=aveqg*fac_top*virt5_ZHtop(igqqLL9,za,zb)
         gqZqLR=aveqg*fac_top*virt5_ZHtop(igqqLR9,za,zb)
         gqZqRL=aveqg*fac_top*virt5_ZHtop(igqqRL9,za,zb)
         gqZqRR=aveqg*fac_top*virt5_ZHtop(igqqRR9,za,zb)
         
         if (toponly .eqv. .false.) then
            gqZqLL=gqZqLL+aveqg*fac*virt5_VH(igqqLL9,za,zb)
            gqZqLR=gqZqLR+aveqg*fac*virt5_VH(igqqLR9,za,zb)
            gqZqRL=gqZqRL+aveqg*fac*virt5_VH(igqqRL9,za,zb)
            gqZqRR=gqZqRR+aveqg*fac*virt5_VH(igqqRR9,za,zb)
         endif
         
         gqbZqbRL=gqZqLL
         gqbZqbRR=gqZqLR
         gqbZqbLL=gqZqRL
         gqbZqbLR=gqZqRR
         
         qgZqLL=aveqg*fac_top*virt5_ZHtop(iqgqLL9,za,zb)
         qgZqLR=aveqg*fac_top*virt5_ZHtop(iqgqLR9,za,zb)
         qgZqRL=aveqg*fac_top*virt5_ZHtop(iqgqRL9,za,zb)
         qgZqRR=aveqg*fac_top*virt5_ZHtop(iqgqRR9,za,zb)
         
         if (toponly .eqv. .false.) then
            qgZqLL=qgZqLL+aveqg*fac*virt5_VH(iqgqLL9,za,zb)
            qgZqLR=qgZqLR+aveqg*fac*virt5_VH(iqgqLR9,za,zb)
            qgZqRL=qgZqRL+aveqg*fac*virt5_VH(iqgqRL9,za,zb)
            qgZqRR=qgZqRR+aveqg*fac*virt5_VH(iqgqRR9,za,zb)
         endif
         
         qbgZqbRL=qgZqLL
         qbgZqbRR=qgZqLR
         qbgZqbLL=qgZqRL
         qbgZqbLR=qgZqRR
      endif

      zsin2winv=zL(2)-zR(2)

      do j=-nflav,nflav
      do k=-nflav,nflav
      if( j .ne. 0 .and. k .ne. 0 .and. j .ne. -k) goto 19

      if     ((j == 0) .and. (k == 0)) then
          msq(j,k)=0._dp
      elseif ((j > 0) .and. (k < 0)) then
          msq(j,k)=+abs(zL(j)*zl1*prop)**2*qqbZgLL
     &             +abs(zR(j)*zr1*prop)**2*qqbZgRR
     &             +abs(zL(j)*zr1*prop)**2*qqbZgLR
     &             +abs(zR(j)*zl1*prop)**2*qqbZgRL
     &             -subuv*msq0(j,k)
     &             +real(zsin2winv*zl1*prop*(qqbZgLLax*conjg(zL(j)*zl1*prop)+qqbZgRLax*conjg(zR(j)*zl1*prop)),dp)
     &             +real(zsin2winv*zr1*prop*(qqbZgRRax*conjg(zR(j)*zr1*prop)+qqbZgLRax*conjg(zL(j)*zr1*prop)),dp)
     &             +real(conjg(zL(j)*zl1*prop)*(Q(j)*q1+zL(j)*zl1*proptop)*qqbZgLLtop,dp)
     &             +real(conjg(zR(j)*zr1*prop)*(Q(j)*q1+zR(j)*zr1*proptop)*qqbZgRRtop,dp)
     &             +real(conjg(zL(j)*zr1*prop)*(Q(j)*q1+zL(j)*zr1*proptop)*qqbZgLRtop,dp)
     &             +real(conjg(zR(j)*zl1*prop)*(Q(j)*q1+zR(j)*zl1*proptop)*qqbZgRLtop,dp)
      elseif ((j < 0) .and. (k > 0)) then
          msq(j,k)=+abs(zL(k)*zl1*prop)**2*qbqZgLL
     &             +abs(zR(k)*zr1*prop)**2*qbqZgRR
     &             +abs(zL(k)*zr1*prop)**2*qbqZgLR
     &             +abs(zR(k)*zl1*prop)**2*qbqZgRL
     &             -subuv*msq0(j,k)
     &             +real(zsin2winv*zl1*prop*(qbqZgLLax*conjg(zL(k)*zl1*prop)+qbqZgRLax*conjg(zR(k)*zl1*prop)),dp)
     &             +real(zsin2winv*zr1*prop*(qbqZgRRax*conjg(zR(k)*zr1*prop)+qbqZgLRax*conjg(zL(k)*zr1*prop)),dp)
     &             +real(conjg(zL(k)*zl1*prop)*(Q(k)*q1+zL(k)*zl1*proptop)*qbqZgLLtop,dp)
     &             +real(conjg(zR(k)*zr1*prop)*(Q(k)*q1+zR(k)*zr1*proptop)*qbqZgRRtop,dp)
     &             +real(conjg(zL(k)*zr1*prop)*(Q(k)*q1+zL(k)*zr1*proptop)*qbqZgLRtop,dp)
     &             +real(conjg(zR(k)*zl1*prop)*(Q(k)*q1+zR(k)*zl1*proptop)*qbqZgRLtop,dp)
      elseif ((j > 0) .and. (k == 0)) then
          msq(j,k)=+abs(zL(j)*zl1*prop)**2*qgZqLL
     &             +abs(zR(j)*zr1*prop)**2*qgZqRR
     &             +abs(zL(j)*zr1*prop)**2*qgZqLR
     &             +abs(zR(j)*zl1*prop)**2*qgZqRL
     &             -subuv*msq0(j,k)
     &             +real(zsin2winv*zl1*prop*(qgZqLLax*conjg(zL(j)*zl1*prop)+qgZqRLax*conjg(zR(j)*zl1*prop)),dp)
     &             +real(zsin2winv*zr1*prop*(qgZqRRax*conjg(zR(j)*zr1*prop)+qgZqLRax*conjg(zL(j)*zr1*prop)),dp)
     &             +real(conjg(zL(j)*zl1*prop)*(Q(j)*q1+zL(j)*zl1*proptop)*qgZqLLtop,dp)
     &             +real(conjg(zR(j)*zr1*prop)*(Q(j)*q1+zR(j)*zr1*proptop)*qgZqRRtop,dp)
     &             +real(conjg(zL(j)*zr1*prop)*(Q(j)*q1+zL(j)*zr1*proptop)*qgZqLRtop,dp)
     &             +real(conjg(zR(j)*zl1*prop)*(Q(j)*q1+zR(j)*zl1*proptop)*qgZqRLtop,dp)
      elseif ((j < 0) .and. (k == 0)) then
          msq(j,k)=+abs(zL(-j)*zl1*prop)**2*qbgZqbLL
     &             +abs(zR(-j)*zr1*prop)**2*qbgZqbRR
     &             +abs(zL(-j)*zr1*prop)**2*qbgZqbLR
     &             +abs(zR(-j)*zl1*prop)**2*qbgZqbRL
     &             -subuv*msq0(j,k)
     &             +real(zsin2winv*zl1*prop*(qbgZqbLLax*conjg(zL(-j)*zl1*prop)+qbgZqbRLax*conjg(zR(-j)*zl1*prop)),dp)
     &             +real(zsin2winv*zr1*prop*(qbgZqbRRax*conjg(zR(-j)*zr1*prop)+qbgZqbLRax*conjg(zL(-j)*zr1*prop)),dp)
     &             +real(conjg(zL(-j)*zl1*prop)*(Q(-j)*q1+zL(-j)*zl1*proptop)*qbgZqbLLtop,dp)
     &             +real(conjg(zR(-j)*zr1*prop)*(Q(-j)*q1+zR(-j)*zr1*proptop)*qbgZqbRRtop,dp)
     &             +real(conjg(zL(-j)*zr1*prop)*(Q(-j)*q1+zL(-j)*zr1*proptop)*qbgZqbLRtop,dp)
     &             +real(conjg(zR(-j)*zl1*prop)*(Q(-j)*q1+zR(-j)*zl1*proptop)*qbgZqbRLtop,dp)
      elseif ((j == 0) .and. (k > 0)) then
          msq(j,k)=+abs(zL(k)*zl1*prop)**2*gqZqLL
     &             +abs(zR(k)*zr1*prop)**2*gqZqRR
     &             +abs(zL(k)*zr1*prop)**2*gqZqLR
     &             +abs(zR(k)*zl1*prop)**2*gqZqRL
     &             -subuv*msq0(j,k)
     &             +real(zsin2winv*zl1*prop*(gqZqLLax*conjg(zL(k)*zl1*prop)+gqZqRLax*conjg(zR(k)*zl1*prop)),dp)
     &             +real(zsin2winv*zr1*prop*(gqZqRRax*conjg(zR(k)*zr1*prop)+gqZqLRax*conjg(zL(k)*zr1*prop)),dp)
     &             +real(conjg(zL(k)*zl1*prop)*(Q(k)*q1+zL(k)*zl1*proptop)*gqZqLLtop,dp)
     &             +real(conjg(zR(k)*zr1*prop)*(Q(k)*q1+zR(k)*zr1*proptop)*gqZqRRtop,dp)
     &             +real(conjg(zL(k)*zr1*prop)*(Q(k)*q1+zL(k)*zr1*proptop)*gqZqLRtop,dp)
     &             +real(conjg(zR(k)*zl1*prop)*(Q(k)*q1+zR(k)*zl1*proptop)*gqZqRLtop,dp)
      elseif ((j == 0) .and. (k < 0)) then
          msq(j,k)=+abs(zL(-k)*zl1*prop)**2*gqbZqbLL
     &             +abs(zR(-k)*zr1*prop)**2*gqbZqbRR
     &             +abs(zL(-k)*zr1*prop)**2*gqbZqbLR
     &             +abs(zR(-k)*zl1*prop)**2*gqbZqbRL
     &             -subuv*msq0(j,k)
     &             +real(zsin2winv*zl1*prop*(gqbZqbLLax*conjg(zL(-k)*zl1*prop)+gqbZqbRLax*conjg(zR(-k)*zl1*prop)),dp)
     &             +real(zsin2winv*zr1*prop*(gqbZqbRRax*conjg(zR(-k)*zr1*prop)+gqbZqbLRax*conjg(zL(-k)*zr1*prop)),dp)
     &             +real(conjg(zL(-k)*zl1*prop)*(Q(-k)*q1+zL(-k)*zl1*proptop)*gqbZqbLLtop,dp)
     &             +real(conjg(zR(-k)*zr1*prop)*(Q(-k)*q1+zR(-k)*zr1*proptop)*gqbZqbRRtop,dp)
     &             +real(conjg(zL(-k)*zr1*prop)*(Q(-k)*q1+zL(-k)*zr1*proptop)*gqbZqbLRtop,dp)
     &             +real(conjg(zR(-k)*zl1*prop)*(Q(-k)*q1+zR(-k)*zl1*proptop)*gqbZqbRLtop,dp)
      endif
      
   19 continue
      enddo
      enddo

      return
      end
