!  Copyright (C) 2019-2021, respective authors of MCFM.
!
!  This program is free software: you can redistribute it and/or modify it under
!  the terms of the GNU General Public License as published by the Free Software
!  Foundation, either version 3 of the License, or (at your option) any later
!  version.
!
!  This program is distributed in the hope that it will be useful, but WITHOUT ANY
!  WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS FOR A
!  PARTICULAR PURPOSE. See the GNU General Public License for more details.
!
!  You should have received a copy of the GNU General Public License along with
!  this program. If not, see <http://www.gnu.org/licenses/>
 
      subroutine qqb_zgam_v(p,msqv)
      implicit none
      include 'src/Inc/types.f'
      
C----Author R.K.Ellis October 2002
C====Virtual corrections to
c     q(-p1)+qbar(-p2)-->e^-(p3)+e-(p4)+gamma(p5)

      include 'src/Inc/constants.f'
      include 'src/Inc/nf.f'
      include 'src/Inc/mxpart.f'
      include 'src/Inc/cplx.h'
      include 'src/Inc/scheme.f'
      include 'src/Inc/zprods_decl.f'
      include 'src/Inc/qcdcouple.f'
      include 'src/Inc/ewcouple.f'
      include 'src/Inc/zcouple_cms.f'
      include 'src/Inc/blha.f'
      integer:: j,h12,h34,h5
      real(dp):: msqv(-nf:nf,-nf:nf),p(mxpart,4),
     & qbq(2),qqb(2)
      real(dp):: fac
      complex(dp):: qbqlo(2,2,2,2),qqblo(2,2,2,2),
     &              qbqnlo(2,2,2,2),qqbnlo(2,2,2,2)
      integer,parameter::jj(-nf:nf)=(/-1,-2,-1,-2,-1,0,1,2,1,2,1/)

      scheme='dred'

      call spinoru(5,p,za,zb)
      if (useblha == 0) then
      call zvirtamps(1,2,3,4,5,za,zb,qbqnlo,qbqlo)
      endif
      call zvirtamps(2,1,3,4,5,za,zb,qqbnlo,qqblo)

      fac=cf*ason2pi*aveqq*8._dp*abs(zesq)**3*xn
      do j=1,2
      qbq(j)=0._dp
      qqb(j)=0._dp
      do h12=1,2
      do h34=1,2
      do h5= 1,2
      qbq(j)=qbq(j)
     & +2._dp*Dble(conjg(qbqlo(j,h12,h34,h5))*qbqnlo(j,h12,h34,h5))
      qqb(j)=qqb(j)
     & +2._dp*Dble(conjg(qqblo(j,h12,h34,h5))*qqbnlo(j,h12,h34,h5))
      enddo
      enddo
      enddo
      qbq(j)=fac*qbq(j)
      qqb(j)=fac*qqb(j)
      enddo

      msqv(:,:)=0._dp
      do j=-nf,nf
        if     (j > 0) then
          msqv(j,-j)=qqb(jj(j))
        elseif (j < 0) then
          msqv(j,-j)=qbq(-jj(j))
        endif
      enddo

      return
      end


      subroutine zvirtamps(p1,p2,p3,p4,p5,za,zb,qbqnlo,qbqlo)
      implicit none
      include 'src/Inc/types.f'
      
C  Amplitudes for virtual corrections taken from Dixon,Kunszt,Signer
C  hep-ph/9803250 Eqns 4.6 and 4.7
      include 'src/Inc/constants.f'
      include 'src/Inc/nf.f'
      include 'src/Inc/mxpart.f'
      include 'src/Inc/cplx.h'
      include 'src/Inc/masses.f'
      include 'src/Inc/zprods_decl.f'
      include 'src/Inc/sprods_com.f'
      include 'src/Inc/zcouple.f'
      include 'src/Inc/ewcharge.f'
      include 'src/Inc/zcouple_cms.f'
      integer:: p1,p2,p3,p4,p5,j
      real(dp):: s34
      complex(dp):: qbqlo(2,2,2,2),qbqnlo(2,2,2,2),fagamma,fbgamma,
     & prp34,vpole,vpl

      call zamps(p1,p2,p3,p4,p5,za,zb,qbqlo)

c--   calculate propagator factors
      s34=s(p3,p4)      
      prp34=s34/cplx2((s34-zmass**2),zmass*zwidth)
      vpl=vpole(s(p1,p2))

      do j=1,2

      qbqnlo(j,1,1,2)=
     &  +(Q(j)*(Q(j)*q1+zL(j)*zl1*prp34)
     & *(fagamma(p1,p2,p3,p4,p5,za,zb)
     &  +fbgamma(p1,p2,p3,p4,p5,za,zb)))
     &          +vpl*qbqlo(j,1,1,2)
      qbqnlo(j,1,2,2)=
     &  +(Q(j)*(Q(j)*q1+zL(j)*zr1*prp34)
     & *(fagamma(p1,p2,p4,p3,p5,za,zb)
     &  +fbgamma(p1,p2,p4,p3,p5,za,zb)))
     &          +vpl*qbqlo(j,1,2,2)
      qbqnlo(j,1,1,1)=
     &  +(Q(j)*(Q(j)*q1+zL(j)*zl1*prp34)
     .*(fagamma(p2,p1,p4,p3,p5,zb,za)
     & +fbgamma(p2,p1,p4,p3,p5,zb,za)))
     &          +vpl*qbqlo(j,1,1,1)
      qbqnlo(j,1,2,1)=
     &  +(Q(j)*(Q(j)*q1+zL(j)*zr1*prp34)
     .*(fagamma(p2,p1,p3,p4,p5,zb,za)
     & +fbgamma(p2,p1,p3,p4,p5,zb,za)))
     &          +vpl*qbqlo(j,1,2,1)

      qbqnlo(j,2,2,1)=      
     &  -(Q(j)*(Q(j)*q1+zR(j)*zr1*prp34)
     & *(fagamma(p1,p2,p3,p4,p5,zb,za)
     &  +fbgamma(p1,p2,p3,p4,p5,zb,za)))
     &          +vpl*qbqlo(j,2,2,1)
      qbqnlo(j,2,1,1)=      
     &  -(Q(j)*(Q(j)*q1+zR(j)*zl1*prp34)
     & *(fagamma(p1,p2,p4,p3,p5,zb,za)
     &  +fbgamma(p1,p2,p4,p3,p5,zb,za)))
     &          +vpl*qbqlo(j,2,1,1)
      qbqnlo(j,2,2,2)=      
     &  -(Q(j)*(Q(j)*q1+zR(j)*zr1*prp34)
     & *(fagamma(p2,p1,p4,p3,p5,za,zb)
     &  +fbgamma(p2,p1,p4,p3,p5,za,zb)))
     &          +vpl*qbqlo(j,2,2,2)
      qbqnlo(j,2,1,2)=      
     &  -(Q(j)*(Q(j)*q1+zR(j)*zl1*prp34)
     & *(fagamma(p2,p1,p3,p4,p5,za,zb)
     &  +fbgamma(p2,p1,p3,p4,p5,za,zb)))
     &          +vpl*qbqlo(j,2,1,2)
      enddo
      return
      end
