!  Copyright (C) 2019-2021, respective authors of MCFM.
!
!  This program is free software: you can redistribute it and/or modify it under
!  the terms of the GNU General Public License as published by the Free Software
!  Foundation, either version 3 of the License, or (at your option) any later
!  version.
!
!  This program is distributed in the hope that it will be useful, but WITHOUT ANY
!  WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS FOR A
!  PARTICULAR PURPOSE. See the GNU General Public License for more details.
!
!  You should have received a copy of the GNU General Public License along with
!  this program. If not, see <http://www.gnu.org/licenses/>
 
      subroutine gg_hg_v(p,msq)
        implicit none
        include 'src/Inc/types.f'
        include 'src/Inc/mxpart.f'
        include 'src/Inc/masses.f'
        include 'src/Inc/hdecaymode.f'
        include 'src/Inc/nf.f'

        real(dp), intent(in) :: p(mxpart,4)
        real(dp), intent(out) :: msq(-nf:nf,-nf:nf)

        real(dp) :: dotvec, s34
        real(dp) :: hdecay,msqhgamgam
        integer, parameter :: iglue = 5

        s34 = dotvec(p(3,:)+p(4,:),p(3,:)+p(4,:))

        hdecay = 0._dp
        msq = 0._dp

        if (hdecaymode == 'tlta') then
            call htautaudecay(p,3,4,hdecay)
        elseif (hdecaymode == 'bqba') then
            call hbbdecay(p,3,4,hdecay)
        elseif (hdecaymode == 'gaga') then
            hdecay=msqhgamgam(s34)
        else
          write(6,*) 'Unimplemented process in gg_hg_v'
          call abort
        endif

        hdecay=hdecay/((s34-hmass**2)**2+(hmass*hwidth)**2)

        call gg_hg_v_nodecay(p,iglue,msq)

        msq = msq*hdecay

      end subroutine

      subroutine gg_hg_zgam_v(p,msq)
        implicit none
        include 'src/Inc/types.f'
        include 'src/Inc/mxpart.f'
        include 'src/Inc/masses.f'
        include 'src/Inc/hdecaymode.f'
        include 'src/Inc/nf.f'

        real(dp), intent(in) :: p(mxpart,4)
        real(dp), intent(out) :: msq(-nf:nf,-nf:nf)

        real(dp) :: dotvec, hdecay
        real(dp) :: shsq, HZgamMSQ
        integer, parameter :: iglue = 6

        hdecay = 0._dp
        
        shsq = dotvec(p(3,:)+p(4,:)+p(5,:), p(3,:)+p(4,:)+p(5,:))
        hdecay = HZgamMSQ(3,4,5)
        hdecay = hdecay/((shsq-hmass**2)**2+(hmass*hwidth)**2)

        call gg_hg_v_nodecay(p,iglue,msq)

        msq = msq*hdecay

      end subroutine
      
      subroutine gg_hg_v_nodecay(p,iglue,msq)
      implicit none
      include 'src/Inc/types.f'
      
      include 'src/Inc/constants.f'
      include 'src/Inc/nf.f'
      include 'src/Inc/mxpart.f'
      include 'src/Inc/masses.f'
      include 'src/Inc/ewcouple.f'
      include 'src/Inc/qcdcouple.f'
      include 'src/Inc/sprods_com.f'
      include 'src/Inc/scheme.f'
      include 'src/Inc/blha.f'
C     (Taken from Ravindran, Smith, van Neerven hep-ph/0201114)
C     Modified by overall factors

      real(dp), intent(in) :: p(mxpart,4)
      integer, intent(in) :: iglue
      real(dp), intent(out) :: msq(-nf:nf,-nf:nf)

      integer:: j,k
      real(dp):: ss,tt,uu,
     & virtgg,virtqa,virtaq,virtqg,virtgq,Asq,fac

      scheme='tH-V'

      call dotem(iglue,p,s)
      ss=s(1,2)
      tt=s(1,iglue)
      uu=s(2,iglue)

      Asq=(as/(3._dp*pi))**2/vevsq

      fac=ason2pi*Asq*gsq

      if (useblha == 0) then
        call hjetfill(ss,tt,uu,virtgg,virtqa,virtaq,virtqg,virtgq)
      else
        if (blhatype == 0) then
          call hjetfillgg(ss,tt,uu,virtgg)
        endif
        if (blhatype == 1) then
          call hjetfillqa(ss,uu,tt,virtaq)
        endif
      endif

      do j=-nf,nf
      do k=-nf,nf
      msq(j,k)=0._dp
      if ((j==0).and.(k==0)) msq(j,k)=avegg*fac*virtgg
      if ((j>0).and.(k==-j)) msq(j,k)=aveqq*fac*virtqa
      if ((j<0).and.(k==-j)) msq(j,k)=aveqq*fac*virtaq
      if ((j==0).and.(k.ne.0)) msq(j,k)=aveqg*fac*virtgq
      if ((j.ne.0).and.(k==0)) msq(j,k)=aveqg*fac*virtqg
      enddo
      enddo

      return
      end
