!  Copyright (C) 2019-2021, respective authors of MCFM.
!
!  This program is free software: you can redistribute it and/or modify it under
!  the terms of the GNU General Public License as published by the Free Software
!  Foundation, either version 3 of the License, or (at your option) any later
!  version.
!
!  This program is distributed in the hope that it will be useful, but WITHOUT ANY
!  WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS FOR A
!  PARTICULAR PURPOSE. See the GNU General Public License for more details.
!
!  You should have received a copy of the GNU General Public License along with
!  this program. If not, see <http://www.gnu.org/licenses/>
 
      function aqmpC12x34(p1,p2,p3,p4,za,zb,msqCoeff)
      implicit none
!     This is a calculation for the process 1_qb^+ 2_q^- 3_g^- 4_g^+ +H
!     Implementation of Eq.~(9.5) from arXiv:2002.04018 v2
      include 'src/Inc/types.f'
      include 'src/Inc/mxpart.f'
      include 'src/Inc/zprods_decl.f'
      include 'src/ggHgg_mass/mtsq.f'
      integer p1,p2,p3,p4
      complex(dp)::aqmpC12x34,aqmpC12x34m0,aqmpC12x34m2,msqCoeff

      msqCoeff=aqmpC12x34m2(p1,p2,p3,p4,za,zb)
      aqmpC12x34=aqmpC12x34m0(p1,p2,p3,p4,za,zb)+mtsq*msqCoeff
      return
      end

      function aqmpC12x34m0(p1,p2,p3,p4,za,zb)
      implicit none
!     This is a calculation for the process 1_qb^+ 2_q^- 3_g^- 4_g^+ +H
!     Implementation of Eq.~(9.5) from arXiv:2002.04018 v2
      include 'src/Inc/types.f'
      include 'src/Inc/mxpart.f'
      include 'src/Inc/zprods_decl.f'
      include 'src/Inc/sprods_com.f'
      integer p1,p2,p3,p4
      real(dp)::s12,s13,s14,s23,s24,s34,lambda,s123,s124,s1234,lam
      complex(dp):: aqmpC12x34m0,aqmpC12x34m0unsym,zab2
      zab2(p1,p2,p3,p4)=za(p1,p2)*zb(p2,p4)+za(p1,p3)*zb(p3,p4)
      lambda(s12,s13,s14)=(s12-s13-s14)**2-4*s13*s14                          
      s12=s(p1,p2)
      s13=s(p1,p3)
      s14=s(p1,p4)
      s23=s(p2,p3)
      s24=s(p2,p4)
      s34=s(p3,p4)
      s123=s12+s13+s23
      s124=s12+s14+s24
      s1234=s12+s13+s14+s23+s24+s34
      lam=lambda(s1234,s12,s34)

      aqmpC12x34m0=
     & 8*(s124-s123)*(s12+s34+2*s13+2*s23)
     & *za(p2,p4)*zb(p1,p3)*zab2(p3,p1,p2,p4)/(zab2(p4,p1,p2,p3)**2*lam) 

     & +((9*s13-7*s23-s14-s24+4*s34)*za(p2,p4)*zb(p1,p4)
     &  -(9*s14-7*s24-s13-s23+4*s34)*za(p2,p3)*zb(p1,p3))
     & /(zab2(p4,p1,p2,p3)**2)

     & +12*s1234*((s13+s23)**2-(s14+s24)**2)
     & *zab2(p2,p3,p4,p1)*zab2(p3,p1,p2,p4)
     & /(zab2(p4,p1,p2,p3)*lam**2)

     & +4*zab2(p3,p1,p2,p4)/(zab2(p4,p1,p2,p3)*lam)
     & *((3*s12+3*s34+4*s13+4*s23+4*s14)*za(p2,p3)*zb(p1,p3)
     &  -(3*s12+3*s34+4*s13+4*s14+4*s24)*za(p2,p4)*zb(p1,p4))

     & -24*zb(p1,p3)*za(p2,p4)*zab2(p3,p1,p2,p4)**2
     & /(zab2(p4,p1,p2,p3)*lam)
     & -8*zb(p1,p4)*za(p2,p3)*zab2(p3,p1,p2,p4)/lam                                     
     & +8*zb(p1,p4)*za(p2,p3)/zab2(p4,p1,p2,p3)

      aqmpC12x34m0=aqmpC12x34m0
     & +aqmpC12x34m0unsym(p1,p2,p3,p4,za,zb)
     & -aqmpC12x34m0unsym(p2,p1,p4,p3,zb,za)                         
      return
      end

 
      function aqmpC12x34m0unsym(p1,p2,p3,p4,za,zb)
      implicit none
!     This is a calculation for the process 1_qb^+ 2_q^- 3_g^- 4_g^+ +H
!     Implementation of Eq.~(9.5) from arXiv:2002.04018 v2
      include 'src/Inc/types.f'
      include 'src/Inc/mxpart.f'
      include 'src/Inc/zprods_decl.f'
      include 'src/Inc/sprods_com.f'
      integer p1,p2,p3,p4
      real(dp):: s12,s13,s14,s23,s24,s34,s124
      complex(dp)::aqmpC12x34m0unsym,zab2
cc                        
      zab2(p1,p2,p3,p4)=za(p1,p2)*zb(p2,p4)+za(p1,p3)*zb(p3,p4)
                          
      s12=s(p1,p2)
      s13=s(p1,p3)
      s14=s(p1,p4)
      s23=s(p2,p3)
      s24=s(p2,p4)
      s34=s(p3,p4)
                          
      s124=s12+s14+s24
                          
      aqmpC12x34m0unsym=2*za(p2,p4)**2*zb(p3,p4)*(s14+s24)**2
     & /(za(p1,p2)*zab2(p4,p1,p2,p3)**3) 
     & +zb(p1,p3)*za(p2,p4)*(s14+s24)*(4*s124-2*s34)
     & /(zab2(p4,p1,p2,p3)**3) 
     & +((s13+s23)*za(p2,p3)*za(p2,p4)
     & *(s14+s24-s13-s23))
     & /(za(p1,p2)*za(p3,p4)*zab2(p4,p1,p2,p3)**2) 
     & +(+2*za(p2,p3)*za(p2,p4)*zb(p3,p4)*(s14+s24))
     & /(za(p1,p2)*zab2(p4,p1,p2,p3)**2) 
      return
      end

      function aqmpC12x34m2(p1,p2,p3,p4,za,zb)
      implicit none
!     This is a calculation for the process 1_qb^+ 2_q^- 3_g^- 4_g^+ +H
!     Implementation of Eq.~(9.6) from arXiv:2002.04018 v2
      include 'src/Inc/types.f'
      include 'src/Inc/mxpart.f'
      include 'src/Inc/zprods_decl.f'
      integer p1,p2,p3,p4
      complex(dp):: aqmpC12x34m2,aqmpC12x34m2unsym
      aqmpC12x34m2=aqmpC12x34m2unsym(p1,p2,p3,p4,za,zb)
     &            -aqmpC12x34m2unsym(p2,p1,p4,p3,zb,za)                         
      return
      end

      function aqmpC12x34m2unsym(p1,p2,p3,p4,za,zb)
      implicit none
!     This is a calculation for the process 1_qb^+ 2_q^- 3_g^- 4_g^+ +H
!     Implementation of Eq.~(9.6) from arXiv:2002.04018 v2
      include 'src/Inc/types.f'
      include 'src/Inc/mxpart.f'
      include 'src/Inc/zprods_decl.f'
      include 'src/Inc/sprods_com.f'
      integer p1,p2,p3,p4
      real(dp):: s12,s13,s14,s23,s24,s34,s1234,lambda,lam
      complex(dp)::aqmpC12x34m2unsym,zab2
cc                        
      zab2(p1,p2,p3,p4)=za(p1,p2)*zb(p2,p4)+za(p1,p3)*zb(p3,p4)
      lambda(s12,s13,s14)=(s12-s13-s14)**2-4*s13*s14                          
                          
      s12=s(p1,p2)
      s13=s(p1,p3)
      s14=s(p1,p4)
      s23=s(p2,p3)
      s24=s(p2,p4)
      s34=s(p3,p4)
      s1234=s12+s13+s14+s23+s24+s34
      lam=lambda(s1234,s12,s34)
      aqmpC12x34m2unsym=
     & +4*za(p2,p3)**2/(za(p1,p2)*za(p3,p4)*zab2(p4,p1,p2,p3))

     & -4*za(p2,p4)**2*zab2(p3,p1,p2,p4)
     & /(za(p1,p2)*za(p3,p4)*zab2(p4,p1,p2,p3)**2)

     & -8*(s13+s23+s14+s24)*za(p2,p3)*za(p2,p4)*zab2(p3,p1,p2,p4)
     & /(za(p1,p2)*za(p3,p4)*zab2(p4,p1,p2,p3)*lam) 

     & -16*zb(p1,p3)*za(p2,p3)*zab2(p3,p1,p2,p4)/(zab2(p4,p1,p2,p3)*lam) 
      return
      end
