!  Copyright (C) 2019-2021, respective authors of MCFM.
!
!  This program is free software: you can redistribute it and/or modify it under
!  the terms of the GNU General Public License as published by the Free Software
!  Foundation, either version 3 of the License, or (at your option) any later
!  version.
!
!  This program is distributed in the hope that it will be useful, but WITHOUT ANY
!  WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS FOR A
!  PARTICULAR PURPOSE. See the GNU General Public License for more details.
!
!  You should have received a copy of the GNU General Public License along with
!  this program. If not, see <http://www.gnu.org/licenses/>
 
      subroutine gg_hgg_mass(p,msq)
        implicit none
        include 'src/Inc/types.f'
        include 'src/Inc/mxpart.f'
        include 'src/Inc/hdecaymode.f'
        include 'src/Inc/nf.f'
        include 'src/Inc/msq_struc.f'
        include 'src/Inc/masses.f'

        real(dp), intent(in) :: p(mxpart,4)
        real(dp), intent(out) :: msq(-nf:nf,-nf:nf)

        integer, parameter :: iglue1 = 5, iglue2 = 6
        real(dp) :: hdecay,s34
        real(dp) :: dotvec, msqhgamgam

        s34 = dotvec(p(3,:)+p(4,:), p(3,:)+p(4,:))

        if (hdecaymode == 'tlta') then
            call htautaudecay(p,3,4,hdecay)
        elseif (hdecaymode == 'bqba') then
            call hbbdecay(p,3,4,hdecay)
        elseif (hdecaymode == 'gaga') then
            hdecay=msqhgamgam(s34)
        else
        write(6,*) 'Unimplemented process in gg_hgg_v'
        stop
        endif
        hdecay=hdecay/((s34-hmass**2)**2+(hmass*hwidth)**2)

        call gg_hgg_mass_nodecay(p,iglue1,iglue2,msq)

        msq = msq * hdecay
        msq_struc = msq_struc * hdecay

      end subroutine


      subroutine gg_hgg_mass_nodecay(p,iglue1,iglue2,msq)
      implicit none
      include 'src/Inc/types.f'
      
c---Matrix element squared averaged over initial colors and spins
c
c     g(-p1)+g(-p2) -->  H(p3)+g(p_iglue1=5)+g(p_iglue2=6) 
c
c   Including the effect of the finite top-quark mass
c
      include 'src/Inc/constants.f'
      include 'src/Inc/nf.f'
      include 'src/Inc/mxpart.f'
      include 'src/Inc/cplx.h'
      include 'src/Inc/masses.f'
      include 'src/Inc/ewcouple.f'
      include 'src/Inc/qcdcouple.f'
      include 'src/Inc/zprods_com.f'
      include 'src/Inc/msq_struc.f'
      include 'src/Inc/nflav.f'
      include 'src/Inc/blha.f'

      real(dp), intent(in) :: p(mxpart,4)
      integer, intent(in) :: iglue1, iglue2
      real(dp), intent(out) :: msq(-nf:nf,-nf:nf)

      integer:: j,k
      real(dp):: pglue(mxpart,4)
      real(dp):: Hgggg,Hqagg,Haqgg,Hgqqg,Hgaag,Hqgqg,Hagag,Hggqa,
     & Hqrqr,Hqqqq,Habab,Haaaa,Hbqbq,Hqaaq,
     & Hqarb,Hqaqa,Hqbqb,Haqbr,Haqaq

c--- Set up momenta for partons
      pglue(1,:)=p(1,:)
      pglue(2,:)=p(2,:)
      pglue(3,:)=p(iglue1,:)
      pglue(4,:)=p(iglue2,:)

C---fill spinor products
      call spinoru(4,pglue,za,zb)

c--- four gluon terms
      if ((useblha==0).or.(blhatype==1)) then
      call hgggg_mass(1,2,3,4,Hgggg)
      endif
      
c--- four quark terms
      if ((useblha==0).or.(blhatype>=7)) then
      call haqaq_mass(1,3,2,4,Hqrqr,Hqqqq)
      Habab=Hqrqr
      Haaaa=Hqqqq
      Hqbqb=Hqrqr
      Hbqbq=Hqrqr
      endif
      
      if (useblha==0) then
      call haqaq_mass(1,2,4,3,Hqarb,Hqaqa)
      Haqbr=Hqarb
      Haqaq=Hqaqa
      endif

c--- two quark two gluon terms
      if (useblha==0) then
      call haqgg_mass(1,2,3,4,Hqagg)
      Haqgg=Hqagg
      endif

      if (useblha==0) then
      call haqgg_mass(1,3,2,4,Hqgqg)
      Hagag=Hqgqg
      endif
      
      if (useblha==0) then
      call haqgg_mass(2,3,1,4,Hgqqg)
      Hgaag=Hgqqg
      endif
      
      if ((useblha==0).or.(blhatype==2)) then
      call haqgg_mass(4,3,1,2,Hggqa)
      endif
      
      do j=-nf,nf
      do k=-nf,nf
      msq(j,k)=zip

      if ((j>0).and.(k>0)) then 
        if (j==k) then
          msq(j,k)=half*aveqq*Hqqqq
        else
          msq(j,k)=aveqq*Hqrqr
        endif
      endif
      
      if ((j<0).and.(k<0)) then 
        if (j==k) then
          msq(j,k)=half*aveqq*Haaaa
        else
          msq(j,k)=aveqq*Habab
        endif
      endif

      if ((j>0).and.(k<0)) then
        if (j==-k) then
          msq(j,k)=aveqq*(half*Hqagg+Hqaqa+real(nflav-1,dp)*Hqarb)
          if (useblha==1) then
             if ((blhafl(5)==0).and.(blhafl(6)==0)) then
                msq(j,k)=aveqq*half*Hqagg
             elseif ((blhafl(5)==blhafl(1)).and.
     &               (blhafl(6)==blhafl(2))) then
                msq(j,k)=aveqq*Hqaqa
             else
                msq(j,k)=aveqq*Hqarb
             endif
          endif
        else
          msq(j,k)=aveqq*Hqbqb
        endif
      endif

      if ((j<0).and.(k>0)) then
        if (j==-k) then
          msq(j,k)=aveqq*(half*Haqgg+Haqaq+real(nflav-1,dp)*Haqbr)
          if (useblha==1) then
             if ((blhafl(5)==0).and.(blhafl(6)==0)) then
                msq(j,k)=aveqq*half*Haqgg
             elseif ((blhafl(5)==blhafl(1)).and.
     &               (blhafl(6)==blhafl(2))) then
                msq(j,k)=aveqq*Haqaq
             else
                msq(j,k)=aveqq*Haqbr
             endif
          endif
        else
          msq(j,k)=aveqq*Hbqbq
        endif
      endif

      if ((j>0).and.(k==0)) then
        msq(j,0)=aveqg*Hqgqg
      endif
      
      if ((j<0).and.(k==0)) then
        msq(j,0)=aveqg*Hagag
      endif

      if ((j==0).and.(k>0)) then
        msq(0,k)=aveqg*Hgqqg
      endif

      if ((j==0).and.(k<0)) then
        msq(0,k)=aveqg*Hgaag
      endif

      if ((j==0).and.(k==0)) then
        msq(0,0)=avegg*(half*Hgggg+real(nflav,dp)*Hggqa)
        if (useblha==1) then
           if ((blhafl(5)==0).and.(blhafl(6)==0)) then
              msq(j,k)=avegg*half*Hgggg
           else
              msq(j,k)=avegg*Hggqa
           endif
        endif
      endif
      
      enddo
      enddo
 
! Print-outs useful for OpenLoops comparison
!      write(6,*) 'Hgggg',avegg*half*Hgggg
!      write(6,*)
!      write(6,*) 'Hqrqr',aveqq*Hqrqr
!      write(6,*) 'Hqqqq',half*aveqq*Hqqqq
!      write(6,*) 'Hqarb',aveqq*Hqarb
!      write(6,*) 'Hqaqa',aveqq*Hqaqa
!      write(6,*)
!     write(6,*) 'Hqagg',aveqq*half*Hqagg
!      write(6,*) 'Hqgqg',aveqg*Hqgqg
!      write(6,*) 'Hgqqg',aveqg*Hgqqg
!      write(6,*) 'Hggqa',avegg*Hggqa
!      write(6,*)

! Over-write for testing stability
!      msq(0,0)=avegg*half*Hgggg
!      msq(1,0)=aveqq*half*Hqagg
!      msq(2,0)=aveqq*Hqaqa
      
      return
      end

 
