!  Copyright (C) 2019-2021, respective authors of MCFM.
!
!  This program is free software: you can redistribute it and/or modify it under
!  the terms of the GNU General Public License as published by the Free Software
!  Foundation, either version 3 of the License, or (at your option) any later
!  version.
!
!  This program is distributed in the hope that it will be useful, but WITHOUT ANY
!  WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS FOR A
!  PARTICULAR PURPOSE. See the GNU General Public License for more details.
!
!  You should have received a copy of the GNU General Public License along with
!  this program. If not, see <http://www.gnu.org/licenses/>
 
      function scpmpmC12x34(p1,p2,p3,p4,za,zb)
      implicit none
!     Scalar loop
!     This is a calculation for the process 1_g^+ 2_g^- 3_g^+ 4_g^- +H
!     Implementation of Eq.~(6.13) from arXiv:2002.04018 v2
      include 'src/Inc/types.f'
      include 'src/Inc/mxpart.f'
      include 'src/Inc/zprods_decl.f'
      include 'src/ggHgg_mass/mtsq.f'
      integer p1,p2,p3,p4
      complex(dp):: scpmpmC12x34,scpmpmC12x34m0,pmpmC12x34m2
cc
      scpmpmC12x34=scpmpmC12x34m0(p1,p2,p3,p4,za,zb)
     & +mtsq*pmpmC12x34m2(p1,p2,p3,p4,za,zb)


      return
      end


      function scpmpmC12x34m0(p1,p2,p3,p4,za,zb)
      implicit none
      include 'src/Inc/types.f'
      include 'src/Inc/mxpart.f'
      include 'src/Inc/zprods_decl.f'
      integer p1,p2,p3,p4
      complex(dp)::scpmpmC12x34m0,scpmpmC12x34m0_unsym
      scpmpmC12x34m0=
     &  scpmpmC12x34m0_unsym(p1,p2,p3,p4,za,zb)
     & +scpmpmC12x34m0_unsym(p3,p4,p1,p2,za,zb)
     & +scpmpmC12x34m0_unsym(p2,p1,p4,p3,zb,za)
     & +scpmpmC12x34m0_unsym(p4,p3,p2,p1,zb,za)
      return 
      end

      function scpmpmC12x34m0_unsym(p1,p2,p3,p4,za,zb)
      implicit none
      include 'src/Inc/types.f'
      include 'src/Inc/mxpart.f'
      include 'src/Inc/zprods_decl.f'
      include 'src/Inc/sprods_com.f'
      integer p1,p2,p3,p4
      real(dp):: s1234,s12,s13,s14,s23,s24,s34,Delta,Pi,zab3123
      complex(dp):: scpmpmC12x34m0_unsym,zab2,zab4123,zab3142,
     & zab1234,zab3124,zab1342,zab2341

!  Statement functions 
      zab2(p1,p2,p3,p4)=za(p1,p2)*zb(p2,p4)+za(p1,p3)*zb(p3,p4)
      Pi(p1,p2,p3,p4)=s(p1,p3)+s(p2,p3)-s(p1,p4)-s(p2,p4)

      s12=s(p1,p2)
      s13=s(p1,p3)
      s14=s(p1,p4)
      s23=s(p2,p3)
      s24=s(p2,p4)
      s34=s(p3,p4)
      s1234 = s(p1,p2)+s(p1,p3)+s(p1,p4)+s(p2,p3)+s(p2,p4)+s(p3,p4)
      zab3123 = s13+s23

      zab1234 = zab2(p1,p2,p3,p4)
      zab3124 = zab2(p3,p1,p2,p4)
      zab3142 = zab2(p3,p1,p4,p2)
      zab1342 = zab2(p1,p3,p4,p2)
      zab2341 = zab2(p2,p3,p4,p1)
      zab4123 = zab2(p4,p1,p2,p3)
      Delta = (s1234-s12-s34)**2-4._dp*s12*s34

      scpmpmC12x34m0_unsym= 
     & 2._dp*(za(p2,p3)**3*zb(p3,p4)*zab3123
     & *(zab3123*zb(p2,p3)-zb(p1,p2)*za(p1,p4)*zb(p3,p4)))
     & /(za(p1,p2)*zab1342*zab3124**3)

     & +2._dp*(za(p2,p3)**2*zb(p3,p4)*zab4123*(-2*s23-s24))
     & /(za(p1,p2)*zab1342*zab3124**2)

     & +2._dp*(zb(p1,p2)*za(p2,p3)**2*zb(p3,p4)
     & *(2*s12*(s23-s14-s34)+2*s13*s23
     & +2*s23**2+s14*s34-s23*s34
     & +2*zb(p1,p2)*za(p1,p3)*za(p2,p4)*zb(p3,p4)
     & ))/(zab1342*zab3124**3)

     & +2._dp*(zb(p1,p4)**2
     & *(za(p1,p4)*za(p2,p4)*(2*(s13-s24)-3*(s34+s14)-4*(s12+s23))
     & -2*za(p1,p3)*zb(p2,p3)*za(p2,p4)**2
     & +3*zb(p1,p3)*za(p1,p4)**2*za(p2,p3)))
     & /(zab1342*zab3124**2)

     & +s14**2*(s12*(6*s13-2*s14+2*s23+2*s24)-s14**2+s23**2)
     & /(zab1342**2*zab3124**2)

     &-8._dp*(s12*s13*s14*s23)
     & /(zab1342**2*zab3124**2)

     &+4._dp*(s14*s1234*zab2341*zab4123)/(zab1342*zab3124*Delta)
     &+4._dp*(za(p1,p2)*zb(p1,p3)*zab2341*zab4123*zab3142)
     & /(zab1342*zab3124*Delta)

     & +(zab1234*zab2341*zab3142*zab4123
     & *(Pi(p4,p3,p2,p1)*Pi(p1,p2,p3,p4)+Delta))
     & /(zab1342**2*zab3124**2*Delta)

     & -3._dp*(s1234*zab2341*zab4123
     & *Pi(p4,p3,p2,p1)*Pi(p1,p2,p3,p4)*(s13+s14+s23+s24))
     & /(2._dp*zab1342*zab3124*Delta**2)

     & +5._dp*(s1234*zab2341*zab4123*(s13+s14+s23+s24))
     & /(2._dp*zab1342*zab3124*Delta)

     & -4._dp*(zab2341*zab4123)/(zab1342*zab3124)
      return
      end

