      subroutine fvoqqx(fo,vc,cl, cr, fmass, key, fvo)
c
c This subroutine computes an off-shell fermion wavefunction from a
c flowing-OUT external fermion and a vector boson.
c
c
c input:
c       complex fo(6)          : flow-out fermion                   <fo|
c       complex vc(6)          : input    vector                      v
c       real    fmass          : mass  of OUTPUT fermion f'
c
c output:
c       complex fvo(6)         : off-shell fermion             <fo,v,f'|
c
      implicit none
      integer key
      double complex fo(6),vc(6),fvo(6),d
      double precision pf(0:3),fmass,pf2,cl,cr

      double precision rZero, rOne
      parameter( rZero = 0.0d0, rOne = 1.0d0 )
      double complex cImag, cZero
      parameter( cImag = ( 0.0d0, 1.0d0 ), cZero = ( 0.0d0, 0.0d0 ) )


      fvo(5) = fo(5)+vc(5)
      fvo(6) = fo(6)+vc(6)

      pf(0) = dble( fvo(5))
      pf(1) = dble( fvo(6))
      pf(2) = dimag(fvo(6))
      pf(3) = dimag(fvo(5))
      pf2 = pf(0)**2-(pf(1)**2+pf(2)**2+pf(3)**2)


      if(key.eq.1) then
      d = -rOne/dcmplx( pf2-fmass**2, 0.d0)
      elseif(key.eq.0) then
      d=-rOne
      else
              print *, "wrong key in helas!"
              stop
      endif

         fvo(1)=-(cr*(pf(0) + pf(3))*
     -     (fo(2)*(vc(2) + cImag*vc(3)) + 
     -       fo(1)*(-vc(1) + vc(4)))) + 
     -  cr*(pf(1) + cImag*pf(2))*
     -   (fo(1)*(-vc(2) + cImag*vc(3)) + 
     -     fo(2)*(vc(1) + vc(4))) + 
     -  cl*fmass*(fo(4)*(vc(2) + cImag*vc(3)) + 
     -     fo(3)*(vc(1) + vc(4)))

         fvo(2)=cl*fmass*(fo(3)*(vc(2) - cImag*vc(3)) + 
     -     fo(4)*(vc(1) - vc(4))) - 
     -  cr*(pf(1) - cImag*pf(2))*
     -   (fo(2)*(vc(2) + cImag*vc(3)) + 
     -     fo(1)*(-vc(1) + vc(4))) + 
     -  cr*(pf(0) - pf(3))*
     -   (fo(1)*(-vc(2) + cImag*vc(3)) + 
     -     fo(2)*(vc(1) + vc(4)))

         fvo(3)=-(cr*fmass*(fo(2)*(vc(2) + cImag*vc(3)) + 
     -       fo(1)*(-vc(1) + vc(4)))) + 
     -  cl*(pf(1) + cImag*pf(2))*
     -   (fo(3)*(-vc(2) + cImag*vc(3)) + 
     -     fo(4)*(-vc(1) + vc(4))) + 
     -  cl*(pf(0) - pf(3))*
     -   (fo(4)*(vc(2) + cImag*vc(3)) + 
     -     fo(3)*(vc(1) + vc(4)))

         fvo(4)=cl*(pf(0) + pf(3))*
     -   (fo(3)*(vc(2) - cImag*vc(3)) + 
     -     fo(4)*(vc(1) - vc(4))) + 
     -  cr*fmass*(fo(1)*(-vc(2) + cImag*vc(3)) + 
     -     fo(2)*(vc(1) + vc(4))) + 
     -  cl*(-pf(1) + cImag*pf(2))*
     -   (fo(4)*(vc(2) + cImag*vc(3)) + 
     -     fo(3)*(vc(1) + vc(4)))

         fvo(1)=d*fvo(1)
         fvo(2)=d*fvo(2)
         fvo(3)=d*fvo(3)
         fvo(4)=d*fvo(4)
      return
      end
